#==========================================================
# Casts --
#
#   handling of PostgreSQL casts
#==========================================================
#
namespace eval Casts {}


#----------------------------------------------------------
# ::Casts::new --
#
#   sets up to create a new Cast
#
# Arguments:
#   none
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Casts::new {} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Casts::new


#----------------------------------------------------------
# ::Casts::open --
#
#   passes work to design proc for opening a Cast
#
# Arguments:
#   cast_    name of Cast to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Casts::open {cast_} {

    ::Casts::design $cast_

}; # end proc ::Casts::open


#----------------------------------------------------------
# ::Casts::design --
#
#   opens window in design mode for selected Cast
#
# Arguments:
#   cast_    name of Cast to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Casts::design {cast_} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Casts::new


#----------------------------------------------------------
# ::Casts::introspect --
#
#   Given a castname, returns the SQL needed to recreate it
#
# Arguments:
#   castname_   name of a cast to introspect
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to make this cast
#----------------------------------------------------------
#
proc ::Casts::introspect {castname_ {dbh_ ""}} {

    set insql [::Casts::clone $castname_ $castname_ $dbh_]

    return $insql

}; # end proc ::Casts::introspect


#----------------------------------------------------------
# ::Casts::clone --
#
#   Like introspect, only changes the castname.
#   Borrows heavily from pg_dump.c internals.
#
# Arguments:
#   srccast_    the original cast
#   dstcast_    the clone cast
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to clone this cast
#----------------------------------------------------------
#
proc ::Casts::clone {srccast_ dstcast_ {dbh_ ""}} {

    global CurrentDB

    if {[string match "" $dbh_]} {
        set dbh_ $CurrentDB
    }

    set V [::Database::getPgVersion $dbh_]

    set insql ""

    set srccast_ [string map {\" {}} $srccast_]
    set dstcast_ [string map {\" {}} $dstcast_]

    # split up the source type name and the target type name
    # for both the clone destination and clone source
    set s1 [lindex [split $srccast_ .] 0]
    set t1 [lindex [split $srccast_ .] 1]
    set s2 [lindex [split $dstcast_ .] 0]
    set t2 [lindex [split $dstcast_ .] 1]

    # if we werent provided a clone destination
    # or a bogus clone destination
    if {[string length $t2]==0} {
        set s2 $s1
        set t2 $t1
    }

    set sql ""

    # cast only exists on PG 7.3 and up
    # well, they did exist before, but there wasnt a pg_cast
    # table, it was instead a special case of type
    if {$V>=7.3} {
        set sql "SELECT t1.typname AS castsource,
                        t2.typname AS casttarget,
                        p.proname AS castfunc,
                        p.proargtypes,
                        p.pronargs,
                        c.castcontext
                  FROM pg_catalog.pg_type t1,
                       pg_catalog.pg_type t2,
                       pg_catalog.pg_cast c,
                       pg_catalog.pg_proc p
                 WHERE t1.oid=c.castsource
                   AND t2.oid=c.casttarget
                   AND p.oid=c.castfunc
                   AND t1.typname='$s1'
                   AND t2.typname='$t1'"
    }

    wpg_select $dbh_ $sql rec {
        set insql "CREATE CAST ($s2 AS $t2)"
        if {[string length $rec(castfunc)]==0} {
            append insql "\n\tWITHOUT FUNCTION"
        } else {
            append insql "\n\tWITH FUNCTION $rec(castfunc)"
            append insql " ([::Functions::getParameters $rec(proargtypes) $rec(pronargs) $dbh_])"
        }
        if {[string match "a" $rec(castcontext)]} {
            append insql "\n\tAS ASSIGNMENT"
        # hmm not sure about the EXPLICIT part here, but it
        # should definitely not be AS IMPLICIT for the 'e'
        } elseif {[string match "e" $rec(castcontext)]} {
            append insql "\n\tAS ASSIGNMENT"
        } else {
            append insql "\n\tAS IMPLICIT"
        }
        append insql "\n;"
    }

    return $insql

}; # end proc ::Casts::clone


