#==========================================================
# Conversions --
#
#   handling of PostgreSQL Conversions
#==========================================================
#
namespace eval Conversions {
    variable encodings
}


#----------------------------------------------------------
# ::Conversions::new --
#
#   sets up to create a new Conversion
#
# Arguments:
#   none
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Conversions::new {} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Conversions::new


#----------------------------------------------------------
# ::Conversions::open --
#
#   passes work to design proc for opening a Conversion
#
# Arguments:
#   conversion_    name of Conversion to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Conversions::open {conversion_} {

    ::Conversions::design $conversion_

}; # end proc ::Conversions::open


#----------------------------------------------------------
# ::Conversions::design --
#
#   opens window in design mode for selected Conversion
#
# Arguments:
#   conversion_    name of Conversion to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Conversions::design {conversion_} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Conversions::new


#----------------------------------------------------------
# ::Conversions::introspect --
#
#   Given a conversionname, returns the SQL needed to recreate it
#
# Arguments:
#   convname_   name of a conversion to introspect
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to make this conversion
#----------------------------------------------------------
#
proc ::Conversions::introspect {convname_ {dbh_ ""}} {

    set insql [::Conversions::clone $convname_ $convname_ $dbh_]

    return $insql

}; # end proc ::Conversions::introspect


#----------------------------------------------------------
# ::Conversions::clone --
#
#   Like introspect, only changes the conversionname
#
# Arguments:
#   srcconv_    the original conversion
#   dstconv_    the clone conversion
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to clone this conversion
#----------------------------------------------------------
#
proc ::Conversions::clone {srcconv_ dstconv_ {dbh_ ""}} {

    global CurrentDB

    variable encodings

    if {![info exists encodings(0)]} {
        setEncodings
    }

    if {[string match "" $dbh_]} {
        set dbh_ $CurrentDB
    }

    set V [::Database::getPgVersion $dbh_]

    set insql ""

    set srcconv_ [string map {\" {}} $srcconv_]
    set dstconv_ [string map {\" {}} $dstconv_]

    set sql ""

    if {$V>=7.3} {
        set nsp [lindex [split $srcconv_ .] 0]
        set srcconv_ [lindex [split $srcconv_ .] 1]
        set sql "SELECT n.nspname,
                        c.conforencoding,
                        c.contoencoding,
                        c.conproc,
                        c.condefault
                   FROM pg_catalog.pg_conversion c,
                        pg_catalog.pg_namespace n
                  WHERE c.conname='$srcconv_'
                    AND n.nspname='$nsp'
                    AND c.connamespace=n.oid"
    } else {
        # conversions did not exist pre-7.3
    }

    wpg_select $dbh_ $sql rec {
        set insql [intlmsg "-- CAUTION: These encoding names will vary between systems."]
        append insql "\nCREATE "
        if {$rec(condefault)=="t"} {
            append insql "DEFAULT "
        }
        append insql "CONVERSION [::Database::quoteObject $dstconv_]
\tFOR $encodings($rec(conforencoding))
\tTO $encodings($rec(contoencoding))
\tFROM $rec(conproc)
;"
    }

    return $insql

}; # end proc ::Conversions::clone



#----------------------------------------------------------
# ::Conversions::setEncodings --
#
#   this will create a list to hold all the known encodings
#   the array is found in PG src include/mb/pg_wchar.h
#
#----------------------------------------------------------
#
proc ::Conversions::setEncodings {} {

    variable encodings

    set tmp [split "
	PG_SQL_ASCII,			
	PG_EUC_JP,					
	PG_EUC_CN,					
	PG_EUC_KR,					
	PG_EUC_TW,					
	PG_JOHAB,					
	PG_UTF8,					
	PG_MULE_INTERNAL,			
	PG_LATIN1,					
	PG_LATIN2,					
	PG_LATIN3,					
	PG_LATIN4,					
	PG_LATIN5,					
	PG_LATIN6,					
	PG_LATIN7,					
	PG_LATIN8,					
	PG_LATIN9,					
	PG_LATIN10,					
	PG_WIN1256,					
	PG_TCVN,					
	PG_WIN874,					
	PG_KOI8R,					
	PG_WIN1251,					
	PG_ALT,						
	PG_ISO_8859_5,				
	PG_ISO_8859_6,				
	PG_ISO_8859_7,				
	PG_ISO_8859_8,				
	PG_SJIS,					
	PG_BIG5,					
	PG_GBK,						
	PG_UHC,						
	PG_WIN1250,					
	PG_GB18030,					
	_PG_LAST_ENCODING_			
    " ,]

    set c 0
    foreach i $tmp {
        set encodings($c) [string trim $i]
        incr c
    }

}; # end proc ::Conversions::setEncodings

