#==========================================================
# Indexes --
#
#   handling of PostgreSQL indexes
#==========================================================
#
namespace eval Indexes {}


#----------------------------------------------------------
# ::Indexes::new --
#
#   sets up to create a new index
#
# Arguments:
#   none
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Indexes::new {} {

    showError [intlmsg "Not yet implemented."]
 
    return

}; # end proc ::Indexes::new


#----------------------------------------------------------
# ::Indexes::open --
#
#   passes work to design proc for opening a index
#
# Arguments:
#   index_    name of index to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Indexes::open {index_} {

    ::Indexes::design $index_

}; # end proc ::Indexes::open


#----------------------------------------------------------
# ::Indexes::design --
#
#   opens window in design mode for selected index
#
# Arguments:
#   index_    name of index to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Indexes::design {index_} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Indexes::new


#----------------------------------------------------------
# ::Indexes::introspect --
#
#   Given a indexname, returns the SQL needed to recreate it
#
# Arguments:
#   indexname_  name of a index to introspect
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to make this index
#----------------------------------------------------------
#
proc ::Indexes::introspect {indexname_ {dbh_ ""}} {

    set insql [::Indexes::clone $indexname_ $indexname_ $dbh_]

    return $insql

}; # end proc ::Indexes::introspect


#----------------------------------------------------------
# ::Indexes::clone --
#
#   Like introspect, only changes the indexname
#
# Arguments:
#   srcindex_   the original index
#   destindex_  the clone index
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to clone this index
#
# Remarks:
#   should probably not use a string map here to change the
#   index source table and the index name, instead, it would
#   be better to fully re-create the SQL; however, this
#   should work for all cases where the src and dest names
#   appear _only_ where we expect them in the PG generated
#   CREATE statement
#----------------------------------------------------------
#
proc ::Indexes::clone {srcindex_ destindex_ {dbh_ ""}} {

    global CurrentDB

    if {[string match "" $dbh_]} {
        set dbh_ $CurrentDB
    }

    set V [::Database::getPgVersion $dbh_]

    set insql ""

    set srcindex_ [string map {\" {}} $srcindex_]
    set destindex_ [string map {\" {}} $destindex_]

    # we have to butcher this because the table name is
    # listed in the main window, like it is for triggers
    set srctbl [join [lrange [split $srcindex_ .] 0 end-1] .]
    set srcidx [lindex [split $srcindex_ .] end]
    set dsttbl [join [lrange [split $destindex_ .] 0 end-1] .]
    set dstidx [lindex [split $destindex_ .] end]

    # check for missing dest tbl
    if {[string length $dsttbl]==0} {
        set dsttbl $srctbl
    }

    set sql ""

    if {$V<7.3} {
        set sql "SELECT indexdef
                   FROM pg_indexes
                  WHERE indexname='$srcidx'"
    } else {
        set nsp [lindex [split $srctbl .] 0]
        set srctbl [lindex [split $srctbl .] 1]
        set sql "SELECT indexdef
                   FROM pg_indexes
                  WHERE indexname='$srcidx'
                    AND schemaname='$nsp'"
    }

    wpg_select $dbh_ $sql rec {
        set insql [string map [list $srcidx $dstidx $srctbl $dsttbl] $rec(indexdef)]
    }

    return $insql

}; # end proc ::Indexes::clone


