#==========================================================
# Languages --
#
#   handling of PostgreSQL procedural languages
#==========================================================
#
namespace eval Languages {}


#----------------------------------------------------------
# ::Languages::new --
#
#   sets up to create a new Language
#
# Arguments:
#   none
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Languages::new {} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Languages::new


#----------------------------------------------------------
# ::Languages::open --
#
#   passes work to design proc for opening a Language
#
# Arguments:
#   language_    name of Language to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Languages::open {language_} {

    ::Languages::design $language_

}; # end proc ::Languages::open


#----------------------------------------------------------
# ::Languages::design --
#
#   opens window in design mode for selected Language
#
# Arguments:
#   language_    name of Language to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Languages::design {language_} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Languages::new


#----------------------------------------------------------
# ::Languages::introspect --
#
#   Given a languagename, returns the SQL needed to recreate it
#
# Arguments:
#   langname_   name of a language to introspect
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to make this language
#----------------------------------------------------------
#
proc ::Languages::introspect {langname_ {dbh_ ""}} {

    set insql [::Languages::clone $langname_ $langname_ $dbh_]

    return $insql

}; # end proc ::Languages::introspect


#----------------------------------------------------------
# ::Languages::clone --
#
#   Like introspect, only changes the languagename.
#   Borrows heavily from pg_dump.c internals.
#
# Arguments:
#   srclang_    the original language
#   dstlang_    the clone language
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to clone this language
#----------------------------------------------------------
#
proc ::Languages::clone {srclang_ dstlang_ {dbh_ ""}} {

    global CurrentDB

    if {[string match "" $dbh_]} {
        set dbh_ $CurrentDB
    }

    set V [::Database::getPgVersion $dbh_]

    set insql ""

    set srclang_ [string map {\" {}} $srclang_]
    set dstlang_ [string map {\" {}} $dstlang_]

    set sql ""

    if {$V>=7.3} {
        set sql "SELECT l.lanname,
                        l.lanpltrusted,
                        l.lanplcallfoid,
                        (SELECT p.proname
                           FROM pg_catalog.pg_proc p
                          WHERE p.oid=l.lanplcallfoid)
                        AS handlername,
                        l.lanvalidator,
                        (SELECT p.proname
                           FROM pg_catalog.pg_proc p
                          WHERE p.oid=l.lanvalidator)
                        AS validatorname,
                        l.lanacl
                   FROM pg_catalog.pg_language l
                  WHERE l.lanname='$srclang_'"
    } else {
        set sql "SELECT l.lanname,
                        l.lanpltrusted,
                        l.lanplcallfoid,
                        p.proname AS handlername,
                        '0' AS lanvalidator,
                        '{=U}' AS lanacl
                   FROM pg_language l,
                        pg_proc p
                  WHERE l.lanname='$srclang_'
                    AND l.lanplcallfoid=p.proname"
    }

    wpg_select $dbh_ $sql rec {
        # just in case dstlang got screwed up
        if {[string length $dstlang_]==0} {
            set dstlang $dstlang_
        }

        set insql "CREATE "
        if {$rec(lanpltrusted)=="t"} {
            append insql "TRUSTED "
        }
        append insql "PROCEDURAL LANGUAGE [::Database::quoteObject $dstlang_]"
        append insql "\n\tHANDLER [::Database::quoteObject $rec(handlername)]"
        if {$rec(lanvalidator)!=0} {
            append insql "\n\tVALIDATOR [::Database::quoteObject $rec(validatorname)]"
        }
        append insql "\n;"
        if {[string length $rec(handlername)]==0} {
            set insql "-- THIS IS NOT A USER DEFINED LANGUAGE --"
        }
    }

    return $insql

}; # end proc ::Languages::clone


