#==========================================================
# Reports --
#
#     procedures for retrieving and displaying data
#
#==========================================================
#
namespace eval Reports {
    variable Win
}


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::new {} {

    global PgAcVar

    variable Win

    Window show .pgaw:ReportBuilder:draft
    tkwait visibility .pgaw:ReportBuilder:draft
    Window show .pgaw:ReportBuilder:menu
    tkwait visibility .pgaw:ReportBuilder:menu

    design:init

    set PgAcVar(report,reportname) {}
    set PgAcVar(report,justpreview) 0

}; # end proc ::Reports::new


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::open {reportname} {

    global PgAcVar CurrentDB

    variable Win

    Window show .pgaw:ReportBuilder:draft
    tkwait visibility .pgaw:ReportBuilder:draft
    Window hide .pgaw:ReportBuilder:draft
    Window show .pgaw:ReportBuilder:menu
    tkwait visibility .pgaw:ReportBuilder:menu
    Window hide .pgaw:ReportBuilder:menu
    Window show .pgaw:ReportPreview
    tkwait visibility .pgaw:ReportPreview

    design:init
    set PgAcVar(report,reportname) $reportname
    design:loadReport
    set PgAcVar(report,justpreview) 1
    design:preview

}; # end proc ::Reports::open


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design {reportname} {

    global PgAcVar

    variable ::Reports::Win

    Window show .pgaw:ReportBuilder:draft
    tkwait visibility .pgaw:ReportBuilder:draft
    Window show .pgaw:ReportBuilder:menu
    tkwait visibility .pgaw:ReportBuilder:menu

    design:init
    set PgAcVar(report,reportname) $reportname
    design:loadReport
    set PgAcVar(report,justpreview) 0

}; # end proc ::Reports::design


#----------------------------------------------------------
# introspect --
#
#   Given a reportname, returns the SQL needed to recreate
#   it.
#
# Arguments:
#   reportname_ name of a report to introspect
#   dbh_        an optional database handle
#
# Returns:
#   insql       the INSERT statement to make this graph
#----------------------------------------------------------
#
proc ::Reports::introspect {reportname_ {dbh_ ""}} {

    set insql [::Reports::clone $reportname_ $reportname_ $dbh_]

    return $insql

}; # end proc ::Reports::introspect


#----------------------------------------------------------
# ::Reports::clone --
#
#   Like introspect, only changes the reportname
#
# Arguments:
#   srcreport_  the original report
#   destreport_ the clone report
#   dbh_        an optional database handle
#
# Returns:
#   insql       the INSERT statement to clone this report
#----------------------------------------------------------
#
proc ::Reports::clone {srcreport_ destreport_ {dbh_ ""}} {

    global CurrentDB

    if {[string match "" $dbh_]} {
        set dbh_ $CurrentDB
    }

    set insql ""

    set sql "SELECT reportsource, reportbody, reportprocs, reportoptions, reportfilter, reportorderby
               FROM pga_reports
              WHERE reportname='$srcreport_'"

    wpg_select $dbh_ $sql rec {
        set insql "
            INSERT INTO pga_reports (reportsource, reportbody, reportprocs, reportoptions, reportfilter, reportorderby, reportname)
                 VALUES ('[::Database::quoteSQL $rec(reportsource)]','[::Database::quoteSQL $rec(reportbody)]','$rec(reportprocs)','$rec(reportoptions)','$rec(reportfilter)','$rec(reportorderby)','[::Database::quoteSQL $destreport_]');"
    }

    return $insql

}; # end proc ::Reports::clone


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:close {} {

    global PgAcVar

    variable ::Reports::Win

    catch {Window destroy $Win(draft)}
    catch {Window destroy $Win(menu)}

}; # end proc ::Reports::design:close


#----------------------------------------------------------
# ::Reports::design:drawReportAreas --
#
#   handles when the y-coord of different report areas is
#   changed
#
# Arguments:
#   none
#
# Returns:
#   none
#
# Modifies:
#   position of items in the design window canvas
#----------------------------------------------------------
#
proc ::Reports::design:drawReportAreas {} {

    global PgAcVar

    variable ::Reports::Win

    variable oldcoors
    variable newcoors

    foreach rg $PgAcVar(report,regions) {
        set oldcoors($rg) [lindex [$Win(draft).c coords bg_$rg] 1]
        $Win(draft).c delete bg_$rg
        $Win(draft).c create line \
            0 $PgAcVar(report,y_$rg) \
            5000 $PgAcVar(report,y_$rg) \
            -tags [subst {bg_$rg}]
        set newcoors($rg) [lindex [$Win(draft).c coords bg_$rg] 1]
        $Win(draft).c create rectangle \
            6 [expr {$PgAcVar(report,y_$rg)-3}] \
            12 [expr {$PgAcVar(report,y_$rg)+3}] \
            -fill black \
            -tags [subst {bg_$rg mov reg}]
        $Win(draft).c lower bg_$rg
    }

    design:getSourceFieldsForFilling

    foreach {field x y objid objtype} $PgAcVar(report,prev_fields) {
        set objmoves 0
        set movedist 0
        if {$y<$PgAcVar(report,y_rpthdr)} {
            foreach rg [list rpthdr] {
                if {$oldcoors($rg)!=$newcoors($rg) \
                  && [string length $oldcoors($rg)] > 0 \
                  && [string length $oldcoors($rg)] > 0} {
                    set objmoves 1
                    set movedist [expr {$newcoors($rg)-$oldcoors($rg)}]
                }
            }
        }
        if {$y<$PgAcVar(report,y_pghdr) && $y>$PgAcVar(report,y_rpthdr)} {
            foreach rg [list rpthdr pghdr] {
                if {$oldcoors($rg)!=$newcoors($rg) \
                  && [string length $oldcoors($rg)] > 0 \
                  && [string length $oldcoors($rg)] > 0} {
                    set objmoves 1
                    set movedist [expr {$newcoors($rg)-$oldcoors($rg)}]
                }
            }
        }
        if {$y<$PgAcVar(report,y_detail) && $y>$PgAcVar(report,y_pghdr)} {
            foreach rg [list rpthdr pghdr detail] {
                if {$oldcoors($rg)!=$newcoors($rg) \
                  && [string length $oldcoors($rg)] > 0 \
                  && [string length $oldcoors($rg)] > 0} {
                    set objmoves 1
                    set movedist [expr {$newcoors($rg)-$oldcoors($rg)}]
                }
            }
        }
        if {$y<$PgAcVar(report,y_pgfoo) && $y>$PgAcVar(report,y_detail)} {
            foreach rg [list rpthdr pghdr detail pgfoo] {
                if {$oldcoors($rg)!=$newcoors($rg) \
                  && [string length $oldcoors($rg)] > 0 \
                  && [string length $oldcoors($rg)] > 0} {
                    set objmoves 1
                    set movedist [expr {$newcoors($rg)-$oldcoors($rg)}]
                }
            }
        }
        if {$y<$PgAcVar(report,y_rptfoo) && $y>$PgAcVar(report,y_pgfoo)} {
            foreach rg [list rpthdr pghdr detail pgfoo rptfoo] {
                if {$oldcoors($rg)!=$newcoors($rg) \
                  && [string length $oldcoors($rg)] > 0 \
                  && [string length $oldcoors($rg)] > 0} {
                    set objmoves 1
                    set movedist [expr {$newcoors($rg)-$oldcoors($rg)}]
                }
            }
        }
        if {$objmoves} {
            $Win(draft).c move $objid 0 $movedist
        }
    }

}; # end proc ::Reports::design:drawReportAreas


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:toggleAlignMode {} {

    variable ::Reports::Win

    set bb [$Win(draft).c bbox hili]
    if {[llength $bb]<4} {return}
    if {[$Win(frp).balign cget -text]==[intlmsg "left"]} {
        $Win(frp).balign configure -text [intlmsg "center"]
        $Win(frp).balign configure -anchor n
        $Win(draft).c itemconfigure hili -anchor n
    } elseif {[$Win(frp).balign cget -text]==[intlmsg "center"]} {
        $Win(frp).balign configure -text [intlmsg "right"]
        $Win(frp).balign configure -anchor e
        $Win(draft).c itemconfigure hili -anchor ne
    } else {
        $Win(frp).balign configure -text [intlmsg "left"]
        $Win(frp).balign configure -anchor w
        $Win(draft).c itemconfigure hili -anchor nw
    }
}; # end proc ::Reports::design:toggleAlignMode


#----------------------------------------------------------
# fonts remain an issue to be dealt with
#----------------------------------------------------------
#
proc ::Reports::design:setFont {} {

    variable ::Reports::Win

    set curr_font [$Win(frp).bfont cget -text]
    if {$curr_font==""} {set curr_font "Helvetica 14"}
    set new_font [SelectFont .fontdlg -parent $Win(menu) \
        -font $curr_font -title "Select Font"]
    if {[string length [string trim $new_font]] != 0} {
        set acflo $new_font
    } else {
        set acflo $curr_font
    }    
    set actual_font $acflo
#    set actual_font "-*"
#    append actual_font "-" [font actual $acflo -family]
#    append actual_font "-" [font actual $acflo -weight]
#    append actual_font "-*"
#    append actual_font "-normal"
#    append actual_font "-"
#    append actual_font "-" [font actual $acflo -size]
#    append actual_font "-*"
#    append actual_font "-*"
#    append actual_font "-*"
#    append actual_font "-*"
#    append actual_font "-*"
#    append actual_font "-iso8859-1"

    $Win(frp).bfont configure -text "$actual_font"
    $Win(frp).bfont configure -font "$actual_font"

    design:setObjectFont

}; # end proc ::Reports::design:setFont


#----------------------------------------------------------
# fills in an array with fields that need to be filled in the form
#----------------------------------------------------------
#
proc ::Reports::design:getSourceFieldsForFilling {} {

    global PgAcVar CurrentDB

    variable ::Reports::Win

    set ol [$Win(draft).c find withtag ro]
    set PgAcVar(report,prev_fields) {}

    foreach objid $ol {
        set tags [$Win(draft).c itemcget $objid -tags]
        lappend PgAcVar(report,prev_fields) \
            [string range [lindex $tags [lsearch -glob $tags f-*]] 2 64]
        lappend PgAcVar(report,prev_fields) \
            [lindex [$Win(draft).c coords $objid] 0]
        lappend PgAcVar(report,prev_fields) \
            [lindex [$Win(draft).c coords $objid] 1]
        lappend PgAcVar(report,prev_fields) $objid
        lappend PgAcVar(report,prev_fields) \
            [lindex $tags [lsearch -glob $tags t_*]]
    }

}; # end proc ::Reports::design:getSourceFieldsForFilling


#----------------------------------------------------------
# fills in an array with columns so formulas can access them
#----------------------------------------------------------
#
proc ::Reports::design:getSourceFieldsForPreview {} {

    global PgAcVar CurrentDB

    set PgAcVar(report,source_fields) [::Database::getColumnsList $PgAcVar(report,tablename)]

}; # end proc ::Reports::design:getSourceFieldsForPreview


#----------------------------------------------------------
# fills in the drop box with column names
#----------------------------------------------------------
#
proc ::Reports::design:getSourceFieldsForFieldBox {} {

    global PgAcVar CurrentDB

    variable ::Reports::Win

    # lets really make sure we should have been called
    if {[winfo exists $Win(srcfields)]} {
        # dont allow non-existent tables or views
        #if {[lsearch -exact [concat [Database::getTablesList] \
        #    [Database::getViewsList]] $PgAcVar(report,tablename)]==-1} {
        #    showError [intlmsg "That table or view doesn't exist!"]
        #    return;
        #}
        $Win(srcfields) delete 0 end
        foreach col [::Database::getColumnsList $PgAcVar(report,tablename)] {
            $Win(srcfields) insert end $col
        }
    }

}; # end proc ::Reports::design:getSourceFieldsForFieldBox


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:hasTag {id tg} {

    variable ::Reports::Win

    if {[lsearch [$Win(draft).c itemcget $id -tags] $tg]==-1} {
        return 0
    } else {
        return 1
    }

}; # end proc ::Reports::design:hasTag


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:init {} {

    global PgAcVar

    set PgAcVar(report,xl_auto) 10
    set PgAcVar(report,xf_auto) 10
    set PgAcVar(report,xp_auto) 10
    set PgAcVar(report,xo_auto) 10
    set PgAcVar(report,regions) {rpthdr pghdr detail pgfoo rptfoo}
    set PgAcVar(report,y_rpthdr) 0
    set PgAcVar(report,y_pghdr) 120
    set PgAcVar(report,y_detail) 240
    set PgAcVar(report,y_pgfoo) 345
    set PgAcVar(report,y_rptfoo) 345
    set PgAcVar(report,e_rpthdr) [intlmsg {Report header}]
    set PgAcVar(report,e_pghdr) [intlmsg {Page header}]
    set PgAcVar(report,e_detail) [intlmsg {Detail record}]
    set PgAcVar(report,e_pgfoo) [intlmsg {Page footer}]
    set PgAcVar(report,e_rptfoo) [intlmsg {Report footer}]

    design:drawReportAreas

}; # end proc ::Reports::design:init


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:loadReport {} {

    global PgAcVar CurrentDB

    variable ::Reports::Win

    $Win(draft).c delete all

    wpg_select $CurrentDB "
        SELECT *
          FROM pga_reports
         WHERE reportname='$PgAcVar(report,reportname)'
    " rcd {
        eval $rcd(reportbody)
        set PgAcVar(report,filter) $rcd(reportfilter)
        set PgAcVar(report,orderby) $rcd(reportorderby)
        set PgAcVar(report,labeltext) {}
        set PgAcVar(report,formula) {}
        set PgAcVar(report,picture) {}
    }

    design:changeDraftCoords
    design:getSourceFieldsForFieldBox
    design:drawReportAreas
    design:loadRegionFields

}; # end proc ::Reports::design:loadReport


#----------------------------------------------------------
# get the preview cranking
#----------------------------------------------------------
#
proc ::Reports::design:preview {} {

    global PgAcVar

#    if {[winfo exists .pgaw:ReportPreview]} {
#        tkwait visibility .pgaw:ReportPreview
#        wm withdraw .pgaw:ReportPreview
#    }

    design:previewInit
    set PgAcVar(report,curr_page) 1
    if {$PgAcVar(report,last_page)>0} {
        design:previewPage
    }

#    if {[winfo exists .pgaw:ReportPreview]} {
#        wm deiconify .pgaw:ReportPreview
#        tkwait visibility .pgaw:ReportPreview
#    }

}; # end proc ::Reports::design:preview


#----------------------------------------------------------
# finds the record and page counts
#----------------------------------------------------------
#
proc ::Reports::design:previewInit {} {

    global PgAcVar CurrentDB

    variable ::Reports::Win

    # we need to see what we are doing so show the window
    Window show .pgaw:ReportPreview
    # this makes sure we have values for the page width/height
    design:changePreviewCoords
    # set up the fields we need to fill with data
    design:getSourceFieldsForFilling
    # set up all the source fields - needed for formulas
    design:getSourceFieldsForPreview

    set sql "SELECT count(*) AS cnt
               FROM $PgAcVar(report,tablename)"
    set res [wpg_exec $CurrentDB $sql]

    set PgAcVar(report,prev_num_recs) [lindex [pg_result $res -getTuple 0] 0]
    pg_result $res -clear
    
    # set up the array of pages and records per page as it will vary
    # set PgAcVar(report,page,recs)

    # get number of detail regions per page (screw report head/foot for now)
    # first: page height - (page header height + page footer height)
    set pgdiff [expr {$PgAcVar(report,ph)-(($PgAcVar(report,y_pghdr)-$PgAcVar(report,y_rpthdr))+($PgAcVar(report,y_pgfoo)-$PgAcVar(report,y_detail)))}]
    # second: result of first / detail height
    set PgAcVar(report,max_recs_per_page) [expr {round(double($pgdiff)/double($PgAcVar(report,y_detail)-$PgAcVar(report,y_pghdr)))}]
    # third: worry about columns, thus chnage the result from second
    set PgAcVar(report,cols_per_page) [expr {int(floor(double($PgAcVar(report,pw)/$PgAcVar(report,rw))))}]
    # double check make sure we at least got one column
    if {$PgAcVar(report,cols_per_page)>1} {
        set PgAcVar(report,max_recs_per_page) [expr {$PgAcVar(report,max_recs_per_page)*$PgAcVar(report,cols_per_page)}]
    } else {
        set PgAcVar(report,cols_per_page) 1
    }
    # get number of pages
    set PgAcVar(report,last_page) [expr {int(ceil(double($PgAcVar(report,prev_num_recs))/double($PgAcVar(report,max_recs_per_page))))}]
    set PgAcVar(report,total_page) $PgAcVar(report,last_page)

}; # end proc ::Reports::design:previewInit


#----------------------------------------------------------
# displays one region
#----------------------------------------------------------
#
proc ::Reports::design:previewRegion {x y objid objtype py recfield shown_recs tuples_list} {

    global PgAcVar CurrentDB

    variable ::Reports::Win

# for fields
if {$objtype=="t_f"} {
    $Win(preview).fr.c create text $x [expr {$py+$y}] \
        -text $recfield \
        -font "[$Win(draft).c itemcget $objid -font]" \
        -anchor [$Win(draft).c itemcget $objid -anchor]
}

# for labels
if {$objtype=="t_l"} {
    $Win(preview).fr.c create text $x [expr {$py+$y}] \
    -text [$Win(draft).c itemcget $objid -text] \
    -font "[$Win(draft).c itemcget $objid -font]" -anchor nw
}

# for pictures
if {$objtype=="t_p"} {
    $Win(preview).fr.c create image $x [expr {$py+$y}] \
        -image [image create photo -file [$Win(draft).c \
            itemcget $objid -image]] \
        -anchor nw
}

# for formulas
if {$objtype=="t_o"} {
    # assign each source field to a variable from the tuples
    for {set c 0} {$c<[llength $tuples_list]} {incr c 2} {
        variable [lindex $tuples_list $c] [lindex $tuples_list [expr {$c+1}]]    
    }
    # compute the formula
    set formula_text [$Win(draft).c itemcget $objid -text]
    set formula_result ""
    # is it a function
    if {[regexp -- "^Function::" $formula_text]} {
        set func [string range $formula_text [string length "Function::"] end]
        # this SELECT should probably be moved into the SELECT for the page
        set sql "SELECT [subst $func]"
        set func_paren [string range $func 0 \
            [expr {[string first "(" $func]-1}]]
        wpg_select $CurrentDB $sql frec {
            set formula_result $frec($func_paren)
        }
        $Win(preview).fr.c create text $x [expr {$py+$y}] \
            -text $formula_result \
            -font "[$Win(draft).c itemcget $objid -font]" \
            -anchor nw
    # is it picture data
    } elseif {[regexp -- "^ImageData::" $formula_text]} {
        set imgdata [string range $formula_text \
            [string length "ImageData::"] end]
        $Win(preview).fr.c create image $x [expr {$py+$y}] \
            -image [image create photo -data [subst $imgdata]] -anchor nw
    # is it a pgaccess image
    } elseif {[regexp -- "^PGAImage::" $formula_text]} {
        set pgaimg [string range $formula_text \
            [string length "PGAImage::"] end]
        set imgdata [::Images::get $pgaimg]
        $Win(preview).fr.c create image $x [expr {$py+$y}] \
            -image [image create photo -data [subst $imgdata]] -anchor nw
    # is it a barcode
    } elseif {[regexp -- "^Barcode::" $formula_text]} {
        set ft [subst [string range $formula_text [string length "Barcode::"] end]]
        set height [string trim [lindex [split $ft] 1]]
        set bc [string trim [lindex [split $ft] 2]]
        set bcpad $bc
        for {set i [string length $bc]} {$i < 12} {incr i} {
            set bcpad "0$bcpad"
        }
        set bst [::Barcode::Code $bcpad]
        for {set i 0} {$i < 95} {incr i} {
            if {[string index $bst $i]} {
                $Win(preview).fr.c create rectangle \
                    [expr {$x + 10 + $i}] [expr {$py+$y}] \
                    [expr {$x + 11 + $i}] [expr {$py+$y+$height}] \
                    -fill black \
                    -width 0
            }
        }
    # well it must just be tcl code then
    } else {
        set formula_result [eval $formula_text]
        $Win(preview).fr.c create text $x [expr {$py+$y}] \
            -text $formula_result \
            -font "[$Win(draft).c itemcget $objid -font]" \
            -anchor nw
    }
}

}; # end proc ::Reports::design:previewRegion


#----------------------------------------------------------
# displays the current page
# for now we worry about the page head/foot and detail, not report head/foot
#----------------------------------------------------------
#
proc ::Reports::design:previewPage {} {

    global PgAcVar CurrentDB

    variable ::Reports::Win

set sql ""
set recfield ""

$Win(preview).fr.c delete all

set shown_recs [expr {($PgAcVar(report,curr_page)-1)*$PgAcVar(report,max_recs_per_page)}]

# get data for all the records on this page

set sql "SELECT *
           FROM $PgAcVar(report,tablename)"
if { [string length [string trim $PgAcVar(report,filter)]] > 0} {
    append sql " WHERE $PgAcVar(report,filter)"
}
if { [string length [string trim $PgAcVar(report,orderby)]] > 0} {
    append sql " ORDER BY $PgAcVar(report,orderby)"
}
append sql "
          LIMIT $PgAcVar(report,max_recs_per_page)
         OFFSET [expr {($PgAcVar(report,curr_page)-1)*$PgAcVar(report,max_recs_per_page)}]"
set page_res [wpg_exec $CurrentDB $sql]

# well if there arent any records theres really no point in continuing
if {[pg_result $page_res -numTuples]<1} {return}

# parse the page header
set py $PgAcVar(report,y_rpthdr)
# we only want the first tuple for the page header
pg_result $page_res -tupleArray 0 tuples
foreach {field x y objid objtype} $PgAcVar(report,prev_fields) {
    if {$y < $PgAcVar(report,y_pghdr)} {
        # make sure we line up the region where it was designed to go
        set y [expr {$y-$PgAcVar(report,y_rpthdr)}]
        # looking for fields
        if {$objtype=="t_f"} {
            set recfield [lindex [array get tuples $field] 1]
        } else {
            set recfield ""
        }
        design:previewRegion $x $y $objid $objtype $py \
            $recfield $shown_recs [array get tuples]
    }
}

# Parsing detail group
set shown_detail_recs 0
# loop thru the printing of each column
for {set col 0} {$col<$PgAcVar(report,cols_per_page)} {incr col} {
    set py $PgAcVar(report,y_pghdr)

    # get all the recs on the page
    while {($py < [expr {($PgAcVar(report,ph)-($PgAcVar(report,y_detail)-$PgAcVar(report,y_pghdr)))}]) \
        && ($shown_recs<$PgAcVar(report,prev_num_recs))} {
        # get fields we need to fill
        foreach {field x y objid objtype} $PgAcVar(report,prev_fields) {
            # now lets get some data for a record
            # well this is a little bit of a hack
            # but at least the catch makes sure we get all the records we can
            catch {
                pg_result $page_res -tupleArray $shown_detail_recs tuples
                if {$y > $PgAcVar(report,y_pghdr) \
                    && $y < $PgAcVar(report,y_detail)} {
                    set y [expr {$y-$PgAcVar(report,y_pghdr)}]
                    if {$objtype=="t_f"} {
                        set recfield [lindex [array get tuples $field] 1]
                    } else {
                        set recfield ""
                    }
                    design:previewRegion \
                        [expr {$x+($PgAcVar(report,rw)*$col)}] \
                        $y $objid $objtype $py $recfield $shown_recs \
                        [array get tuples]
                }
            }
        }
        incr shown_recs
        incr shown_detail_recs
        incr py [expr {$PgAcVar(report,y_detail)-$PgAcVar(report,y_pghdr)}]
    }
}

# parse the page footer
# put it in the same place on each page
set py [expr {(($PgAcVar(report,y_detail)-$PgAcVar(report,y_pghdr))*$PgAcVar(report,max_recs_per_page))+$PgAcVar(report,y_pghdr)}]
# get the data for the region, which is the last record on the page
pg_result $page_res \
    -tupleArray [expr {[pg_result $page_res -numTuples]-1}] tuples
foreach {field x y objid objtype} $PgAcVar(report,prev_fields) {
    if {$y > $PgAcVar(report,y_detail) && $y < $PgAcVar(report,y_pgfoo)} {
        set y [expr {$y-$PgAcVar(report,y_detail)}]
        if {$objtype=="t_f"} {
            set recfield [lindex [array get tuples $field] 1]
        } else {
            set recfield ""
        }
        design:previewRegion $x $y $objid $objtype $py $recfield \
            [expr {$shown_recs-1}] [array get tuples]
    }
}

pg_result $page_res -clear

}; # end proc ::Reports::design:previewPage


#----------------------------------------------------------
# opens the print dialog and registers the callback function
#----------------------------------------------------------
#
proc ::Reports::design:print {} {

    ::Printer::init "::Reports::design:printcallback"

}; # end proc ::Reports::design:print


#----------------------------------------------------------
# prints all pages between and including those in the entry boxes
# gets a channel from print dialog and starts feeding it postscript
#----------------------------------------------------------
#
proc ::Reports::design:printcallback {fid} {

    global PgAcVar

    variable ::Reports::Win

    Printer::printStart $fid $PgAcVar(report,pw) $PgAcVar(report,ph) \
        [expr {$PgAcVar(report,last_page)-$PgAcVar(report,curr_page)+1}]

    set start_page $PgAcVar(report,curr_page)

    for {set pgcnt 1} \
        {$PgAcVar(report,curr_page)<=$PgAcVar(report,last_page)} \
        {incr PgAcVar(report,curr_page)} {
            design:previewPage
            Printer::printPage $fid $pgcnt $Win(preview).fr.c
            incr pgcnt
    }

    Printer::printStop $fid

    # reset current page to the page we started printing on
    set PgAcVar(report,curr_page) $start_page
    design:previewPage

}; # end proc ::Reports::design:printcallback


#----------------------------------------------------------
# ::Reports::design:save --
#
#   Saves the report meta-data, or saves a copy of it.
#
# Arguments:
#   copy_   0 default of regular save, 1 for saving a copy
#----------------------------------------------------------
#
proc ::Reports::design:save {{copy_ 0}} {

    global PgAcVar CurrentDB

    variable ::Reports::Win

    # do not allow empty report names
    if {[string length $PgAcVar(report,reportname)]==0} {
        showError [intlmsg "You must supply a name for the report before you can save it!"]
        return
    }

    set prog "set PgAcVar(report,tablename) {$PgAcVar(report,tablename)}"
    append prog " ; set PgAcVar(report,rw) $PgAcVar(report,rw)"
    append prog " ; set PgAcVar(report,rh) $PgAcVar(report,rh)"
    append prog " ; set PgAcVar(report,pw) $PgAcVar(report,pw)"
    append prog " ; set PgAcVar(report,ph) $PgAcVar(report,ph)"
    foreach region $PgAcVar(report,regions) {
        append prog " ; set PgAcVar(report,y_$region) $PgAcVar(report,y_$region)"
    }

    foreach objid [$Win(draft).c find all] {

        if {[$Win(draft).c type $objid]=="text"} {
            set bb [$Win(draft).c bbox $objid]
            if {[$Win(draft).c itemcget $objid -anchor]=="nw"} {
                set x [expr {[lindex $bb 0]+1}]
            } elseif {[$Win(draft).c itemcget $objid -anchor]=="n"} {
                set x [expr {([lindex $bb 2]-[lindex $bb 0])/2}]
            } else {
                set x [expr {[lindex $bb 2]-2}]
            }
            append prog " ; $Win(draft).c create text $x [lindex $bb 1] -font \"[$Win(draft).c itemcget $objid -font]\" -anchor [$Win(draft).c itemcget $objid -anchor] -text {[string map {\' \\'} [$Win(draft).c itemcget $objid -text]]} -tags {[$Win(draft).c itemcget $objid -tags]}"
        }

        if {[$Win(draft).c type $objid]=="image"} {
            set bb [$Win(draft).c bbox $objid]
            if {[$Win(draft).c itemcget $objid -anchor]=="nw"} {
                set x [expr {[lindex $bb 0]+1}]
            } else {
                set x [expr {[lindex $bb 2]-2}]
            }
            append prog " ; image create photo [$Win(draft).c itemcget $objid -image] -file [$Win(draft).c itemcget $objid -image] ; $Win(draft).c create image $x [lindex $bb 1] -anchor [$Win(draft).c itemcget $objid -anchor] -image {[$Win(draft).c itemcget $objid -image]} -tags {[$Win(draft).c itemcget $objid -tags]}"
        }
    }

    if {$copy_ == 0} {
        set sql "
            UPDATE pga_reports
               SET reportsource='$PgAcVar(report,tablename)',
                   reportbody='$prog',
                   reportfilter='$PgAcVar(report,filter)',
                   reportorderby='$PgAcVar(report,orderby)'
             WHERE reportname='$PgAcVar(report,reportname)'"
    } else {
        set sql "
            INSERT INTO pga_reports
                   (reportname,
                   reportsource,
                   reportbody,
                   reportfilter,
                   reportorderby)
            VALUES ('$PgAcVar(report,reportname)',
                    '$PgAcVar(report,tablename)',
                    '$prog',
                    '$PgAcVar(report,filter)'),
                    '$PgAcVar(report,orderby)')"
    }

    sql_exec noquiet $sql

    # refresh right pane
    ::Mainlib::cmd_Reports

}; # end proc ::Reports::design:save


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:editMoveItem {objid sb startx starty} {
    variable ::Reports::Win

    set newpos [$sb cget -text]
    set coord [$Win(draft).c coords $objid]

    if { $startx != -1 } {
        set coord [lreplace $coord 0 0 $newpos]
    } else {
        set coord [lreplace $coord 1 1 $newpos]
    }

    $Win(draft).c coords $objid $coord
}


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:editStart {path row col txt} {

    set colname [$path columncget $col -name]
    set objid [lindex [$path getcolumns id] $row]

    switch $colname {
        x {
            set sb [$path editwinpath]
            $sb configure \
                -range {0 9999 1} \
                -modifycmd [list ::Reports::design:editMoveItem $objid $sb $txt -1]
        } y {
            set sb [$path editwinpath]
            $sb configure \
                -range {0 9999 1} \
                -modifycmd [list ::Reports::design:editMoveItem $objid $sb -1 $txt]
        } default {}
    }
    return $txt
}


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:editEnd {path row col txt} {
    variable ::Reports::Win

    set colname [$path columncget $col -name]
    set objid [lindex [$path getcolumns id] $row]

    switch $colname {
        x {
            set sb [$path editwinpath]
            design:editMoveItem $objid $sb $txt -1
        } y {
            set sb [$path editwinpath]
            design:editMoveItem $objid $sb -1 $txt
        } value {
            $Win(draft).c itemconfigure $objid -text $txt
        } default {}
    }
    return $txt
}

#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:addField {} {

    global PgAcVar

    variable ::Reports::Win

    set fldname [$Win(srcfields) get [$Win(srcfields) curselection]]
    set x $PgAcVar(report,xf_auto)
    set y [expr {$PgAcVar(report,y_pghdr)+5}]
    set newid [$Win(draft).c create text $x $y \
        -text $fldname \
        -tags [subst {f-$fldname t_f rg_detail mov ro}] \
        -anchor nw \
        -font "$PgAcVar(pref,font_normal)"]
    set bb [$Win(draft).c bbox $newid]

    incr PgAcVar(report,xf_auto) [expr {5+[lindex $bb 2]-[lindex $bb 0]}]

    $Win(usedfields) insert end [list $newid $x $y [intlmsg "field"] $fldname]

}; # end proc ::Reports::design:addField


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:addLabel {} {

    global PgAcVar

    variable ::Reports::Win

    set fldname $PgAcVar(report,labeltext)
    set x $PgAcVar(report,xl_auto)
    set y [expr {$PgAcVar(report,y_rpthdr)+5}]
    set newid [$Win(draft).c create text $x $y \
        -text $fldname \
        -tags [subst {t_l mov ro}] \
        -anchor nw \
        -font "$PgAcVar(pref,font_normal)"]
    set bb [$Win(draft).c bbox $newid]

    incr PgAcVar(report,xl_auto) [expr {5+[lindex $bb 2]-[lindex $bb 0]}]

    $Win(usedfields) insert end [list $newid $x $y [intlmsg "label"] $fldname]

}; # end proc ::Reports::design:addLabel


#----------------------------------------------------------
# browse for pictures
#----------------------------------------------------------
#
proc ::Reports::design:browsePicture {} {

    global PgAcVar

    set types {
        {{GIF Files}    {.gif}}
        {{BMP Files}    {.bmp}}
        {{All Files}    *}
    }

    set PgAcVar(report,picture) [tk_getOpenFile \
        -filetypes $types -title "Choose Picture"]

}; # end proc ::Reports::design:browsePicture


#----------------------------------------------------------
# pictures are from files and not the database, maybe this should be different
#----------------------------------------------------------
#
proc ::Reports::design:addPicture {} {

    global PgAcVar

    variable ::Reports::Win

    set fldname $PgAcVar(report,picture)
    if {[file exists $fldname]} {
        set x $PgAcVar(report,xp_auto)
        set y [expr {$PgAcVar(report,y_rpthdr)+5}]
        set newid [$Win(draft).c create image $x $y \
            -image [image create photo $fldname -file $fldname] \
            -tags [subst {t_p mov ro}] -anchor nw]
        set bb [$Win(draft).c bbox $newid]
        incr PgAcVar(report,xp_auto) [expr {5+[lindex $bb 2]-[lindex $bb 0]}]
        $Win(usedfields) insert end [list $newid $x $y [intlmsg "picture"] $fldname]
    }

}; # end proc ::Reports::design:addPicture


#----------------------------------------------------------
# formulas are tcl snippets or functions, could be scripts sometime too
#----------------------------------------------------------
#
proc ::Reports::design:addFormula {} {

    global PgAcVar

    variable ::Reports::Win

    set fldname $PgAcVar(report,formula)
    set x $PgAcVar(report,xo_auto)
    set y [expr {$PgAcVar(report,y_rpthdr)+5}]
    set newid [$Win(draft).c create text $x $y \
        -text $fldname -tags [subst {t_o mov ro}] \
        -anchor nw -font "$PgAcVar(pref,font_normal)"]
    set bb [$Win(draft).c bbox $newid]

    incr PgAcVar(report,xo_auto) [expr {5+[lindex $bb 2]-[lindex $bb 0]}]

    $Win(usedfields) insert end [list $newid $x $y [intlmsg "formula"] $fldname]

}; # end proc ::Reports::design:addFormula


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:setObjectFont {} {

    global PgAcVar

    variable ::Reports::Win

    $Win(draft).c itemconfigure hili \
        -font "[$Win(frp).bfont cget -text]"

}; # end proc ::Reports::design:setObjectFont


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:updateObject {} {

    global PgAcVar

    variable ::Reports::Win

    set objid [$Win(draft).c find withtag hili] 
    if {[design:hasTag $objid t_l]} {
        $Win(draft).c itemconfigure hili \
            -text $PgAcVar(report,labeltext)
    } elseif {[design:hasTag $objid t_o]} {
        $Win(draft).c itemconfigure hili \
            -text $PgAcVar(report,formula)
    }

}; # end proc ::Reports::design:updateObject


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:deleteObject {} {

    variable ::Reports::Win

    set tblrow [$Win(usedfields) curselection]
    set objid [lindex [$Win(usedfields) getcolumns id] $tblrow]
    if {$tblrow!=-1} {
        if {[design:hasTag $objid t_p]} {
            set val "(picture)"
        } else {
            set val [$Win(draft).c itemcget $objid -text]
            if { [string length $val] > 20 } {
                set val [string range $val 0 20]
                append val "..."
            }
        }
        if {[tk_messageBox -title [intlmsg Warning] \
            -parent $Win(draft) -message "[intlmsg {Delete current report object?}] $val" \
            -type yesno -default no]=="no"} return;
        $Win(usedfields) delete [list $tblrow]
    }
    $Win(draft).c delete $objid

}; # end proc ::Reports::design:deleteObject


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:dragMove {w x y} {

    global PgAcVar

    # Showing current region
    if {![info exists PgAcVar(report,regions)]} {return}

    foreach rg $PgAcVar(report,regions) {
        set PgAcVar(report,msg) $PgAcVar(report,e_$rg)
        if {$PgAcVar(report,y_$rg)>$y} break;
    }

    set temp {}

    catch {set temp $PgAcVar(draginfo,objid)}

    if {"$temp" != ""} {
        set dx [expr {$x - $PgAcVar(draginfo,x)}]
        set dy [expr {$y - $PgAcVar(draginfo,y)}]
        if {$PgAcVar(draginfo,region)!=""} {
            set x $PgAcVar(draginfo,x)
            $w move bg_$PgAcVar(draginfo,region) 0 $dy
        } else {
            $w move $PgAcVar(draginfo,objid) $dx $dy
        }
        set PgAcVar(draginfo,x) $x
        set PgAcVar(draginfo,y) $y
    }

}; # end proc ::Reports::design:dragMove


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:dragStart {w x y} {

    global PgAcVar

    variable ::Reports::Win

focus $Win(draft).c
catch {unset draginfo}
set objid {}

# Only movable objects start dragging
foreach id [$w find overlapping $x $y $x $y] {
    if {[design:hasTag $id mov]} {
        set objid $id
        break
    }
}

# mouse resize does update after a click
if {$objid==""} {
    $Win(draft).c configure -cursor watch
    set c [split [winfo geometry $Win(draft).c] x+]
    set PgAcVar(report,rw) [lindex $c 0]
    set PgAcVar(report,rh) [lindex $c 1]
    Reports::design:changeDraftCoords
    return
}

set PgAcVar(draginfo,objid) $objid
set taglist [$Win(draft).c itemcget $objid -tags]
set i [lsearch -glob $taglist bg_*]

if {$i==-1} {
    set PgAcVar(draginfo,region) {}
} else {
    set PgAcVar(draginfo,region) [string range [lindex $taglist $i] 3 64]
}

$Win(draft).c configure -cursor hand1
# dont highlight pictures when moving them, it just wont work
if {![design:hasTag [$Win(draft).c find withtag hili] t_p]} {
    $Win(draft).c itemconfigure \
        [$Win(draft).c find withtag hili] -fill black
}
    $Win(draft).c dtag \
        [$Win(draft).c find withtag hili] hili
    $Win(draft).c addtag hili withtag $PgAcVar(draginfo,objid)
if {![design:hasTag $objid t_p]} {
    $Win(draft).c itemconfigure hili -fill blue
}
set PgAcVar(draginfo,x) $x
set PgAcVar(draginfo,y) $y
set PgAcVar(draginfo,sx) $x
set PgAcVar(draginfo,sy) $y
# Setting font information
if {[$Win(draft).c type hili]=="text"} {
    $Win(frp).bfont configure \
        -text [$Win(draft).c itemcget hili -font]
    if {[$Win(draft).c itemcget $objid -anchor]=="nw"} {
        $Win(frp).balign configure -text [intlmsg "left"]
        $Win(frp).balign configure -anchor w
    } elseif {[$Win(draft).c itemcget $objid -anchor]=="n"} {
        $Win(frp).balign configure -text [intlmsg "center"]
        $Win(frp).balign configure -anchor n
    } else {
        $Win(frp).balign configure -text [intlmsg "right"]
        $Win(frp).balign configure -anchor e
    }
}

if {[design:hasTag $objid t_f]} {
    set PgAcVar(report,info) "Database field"
    design:selectField $objid
}
if {[design:hasTag $objid t_l]} {
    set PgAcVar(report,info) "Label"
    design:selectField $objid
}
if {[design:hasTag $objid t_o]} {
    set PgAcVar(report,info) "Formula"
    design:selectField $objid
}
if {[design:hasTag $objid t_p]} {
    set PgAcVar(report,info) "Picture"
    design:selectField $objid
}

}; # end proc ::Reports::design:dragStart


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:dragStop {x y} {

    global PgAcVar

    variable ::Reports::Win

# when click Close, ql window is destroyed but event ButtonRelease-1 is fired
if {![winfo exists $Win(draft).c]} return;
$Win(draft).c configure -cursor left_ptr
set este {}
catch {set este $PgAcVar(draginfo,objid)}
if {$este==""} return
# Erase information about object beeing dragged
if {$PgAcVar(draginfo,region)!=""} {
    set dy 0
    foreach rg $PgAcVar(report,regions) {
        $Win(draft).c move rg_$rg 0 $dy
        if {$rg==$PgAcVar(draginfo,region)} {
            set dy [expr {$y-$PgAcVar(report,y_$PgAcVar(draginfo,region))}]
        }
        incr PgAcVar(report,y_$rg) $dy
    }
#    .pgaw:ReportBuilder:menu.c move det 0 [expr $y-$PgAcVar(report,y_$PgAcVar(draginfo,region))]
    set PgAcVar(report,y_$PgAcVar(draginfo,region)) $y
    design:drawReportAreas
} else {
    # Check if object beeing dragged is inside the canvas
    set bb [$Win(draft).c bbox $PgAcVar(draginfo,objid)]
    # too far to the left
    if {[lindex $bb 0] < 5} {
        $Win(draft).c move $PgAcVar(draginfo,objid) \
            [expr {5-[lindex $bb 0]}] 0
    }
    # too far to the right
    if {[lindex $bb 0] > [expr {$PgAcVar(report,rw)-10}]} {
        $Win(draft).c move $PgAcVar(draginfo,objid) \
            [expr {$PgAcVar(report,rw)-25-[lindex $bb 0]}] 0
    }
    # too far above
    if {[lindex $bb 1] < 2} {
        $Win(draft).c move $PgAcVar(draginfo,objid) \
            0 [expr {2-[lindex $bb 1]}]
    }
    # too far below canvas
    if {[lindex $bb 1] > [expr {$PgAcVar(report,rh)-10}]} {
        $Win(draft).c move $PgAcVar(draginfo,objid) \
            0 [expr {$PgAcVar(report,rh)-15-[lindex $bb 1]}]
    }
    # too far below report footer
    if {[lindex $bb 1] > [expr {$PgAcVar(report,y_rptfoo)-10}]} {
        $Win(draft).c move $PgAcVar(draginfo,objid) \
            0 [expr {$PgAcVar(report,y_rptfoo)-15-[lindex $bb 1]}]
    }
    design:selectField $PgAcVar(draginfo,objid)
}
set PgAcVar(draginfo,objid) {}
PgAcVar:clean draginfo,*

}; # end proc ::Reports::design:dragStop


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:deleteAllObjects {} {

    variable ::Reports::Win

    if {[tk_messageBox -title [intlmsg Warning] \
        -parent $Win(draft) \
        -message [intlmsg "All report information will be deleted.\n\nProceed ?"] -type yesno -default no]=="yes"} then {
        $Win(draft).c delete all
        design:init
        design:drawReportAreas
    }

}; # end proc ::Reports::design:deleteAllObjects


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:changeDraftCoords {} {

    global PgAcVar

    variable ::Reports::Win

    # make sure we have a report width and height, if not default something
    if {[string length $PgAcVar(report,rw)]==0} {
        set PgAcVar(report,rw) 508
    }

    if {[string length $PgAcVar(report,rh)]==0} {
        set PgAcVar(report,rh) 345
    }

    wm geometry $Win(draft) \
        $PgAcVar(report,rw)x$PgAcVar(report,rh)
    place $Win(draft).c -x 0 -y 0 \
        -width $PgAcVar(report,rw) -height $PgAcVar(report,rh) \
        -anchor nw -bordermode ignore

}; # end proc ::Reports::design:changeDraftCoords


#----------------------------------------------------------
#----------------------------------------------------------
#
proc ::Reports::design:changePreviewCoords {} {

    global PgAcVar

    variable ::Reports::Win

    # make sure we have a page width and height, if not default something
    if {![info exists PgAcVar(report,pw)] || \
        [string length $PgAcVar(report,pw)]<=1} {
        set PgAcVar(report,pw) 508
    }

    if {![info exists PgAcVar(report,ph)] || \
        [string length $PgAcVar(report,ph)]<=1} {
        set PgAcVar(report,ph) 345
    }

    # check the geometry so you dont have to resize in order to print
    set g ""
    set w $PgAcVar(report,pw)
    set h $PgAcVar(report,ph)
    if {$PgAcVar(report,pw) < 350} {
        set w 350
    }
    if {$PgAcVar(report,ph) < 100} {
        set h 100
    }
    append g $w "x" $h
    wm geometry $Win(preview) $g
    place $Win(preview).fr.c -x 0 -y 0 \
        -width $PgAcVar(report,pw) -height $PgAcVar(report,ph) \
        -anchor nw -bordermode ignore
    wm deiconify $Win(preview)

}; # end proc ::Reports::design:changePreviewCoords


#----------------------------------------------------------
# fills the list box for each region with the fields in that region
#----------------------------------------------------------
#
proc ::Reports::design:loadRegionFields {} {

    global PgAcVar

    variable ::Reports::Win

    design:getSourceFieldsForFilling

    foreach {field x y objid objtype} $PgAcVar(report,prev_fields) {
        switch $objtype {
            t_f {
                set type [intlmsg "field"]
                set val [$Win(draft).c itemcget $objid -text]
            } t_l {
                set type [intlmsg "label"]
                set val [$Win(draft).c itemcget $objid -text]
            } t_o {
                set type [intlmsg "formula"]
                set val [$Win(draft).c itemcget $objid -text]
            } t_p {
                set type [intlmsg "picture"]
                set val [$Win(draft).c itemcget $objid -image]
            } default {
                set type unknown
            }
        }
        set x [expr {round($x)}]
        set y [expr {round($y)}]
        $Win(usedfields) insert end [list $objid $x $y $type $val]
    }

    $Win(usedfields) sortbycolumn 0

}; # end proc ::Reports::design:loadRegionFields


#----------------------------------------------------------
# given a y value detemines region item is in and returns it
#----------------------------------------------------------
#
proc ::Reports::design:determineRegion {y} {

    global PgAcVar

    set retval ""

    if {$y<$PgAcVar(report,y_rpthdr)} {
        set retval "rpthdr"
    }
    if {$y<$PgAcVar(report,y_pghdr) && $y>$PgAcVar(report,y_rpthdr)} {
        set retval "pghdr"
    }
    if {$y<$PgAcVar(report,y_detail) && $y>$PgAcVar(report,y_pghdr)} {
        set retval "detail"
    }
    if {$y<$PgAcVar(report,y_pgfoo) && $y>$PgAcVar(report,y_detail)} {
        set retval "pgfoo"
    }
    if {$y<$PgAcVar(report,y_rptfoo) && $y>$PgAcVar(report,y_pgfoo)} {
        set retval "rptfoo"
    }

    return $retval

}; # end proc ::Reports::design:determineRegion


#----------------------------------------------------------
# gets info on the selected field in a region by clicking in the menu window
#----------------------------------------------------------
#
proc ::Reports::design:selectRegionFieldFromMenu {_w _x _y} {
    foreach {w x y} [tablelist::convEventFields $_w $_x $_y] {}
    set cell [$w containingcell $x $y]
    set row [lindex [split $cell ,] 0]
    set objid [lindex [$w getcolumns id] $row]
    design:selectField $objid
}; # end proc ::Reports::design:selectRegionFieldFromMenu


#----------------------------------------------------------
# highlight the field/label/formula, select it in the tablelist
#----------------------------------------------------------
#
proc ::Reports::design:selectField {objid} {

    global PgAcVar

    variable ::Reports::Win

    set rg [design:determineRegion [lindex [$Win(draft).c coords $objid] 1]]

    # highlight the item (in case it isnt already)
    if { ![catch {$Win(draft).c itemcget [$Win(draft).c find withtag hili] -fill}]} {
        $Win(draft).c itemconfigure [$Win(draft).c find withtag hili] -fill black
    }
    $Win(draft).c dtag [$Win(draft).c find withtag hili] hili

    if { ![catch {$Win(draft).c itemcget $objid -fill}]} {
        $Win(draft).c itemconfigure $objid -fill blue
    }
    $Win(draft).c addtag hili withtag $objid

    # clears then selects the item in the listbox (in case it isnt already)
    $Win(usedfields) selection clear [$Win(usedfields) curselection]
    set tblrow [lsearch [$Win(usedfields) getcolumns id] $objid]
    if {$tblrow!=-1} {
        $Win(usedfields) selection set [list $tblrow]
        $Win(usedfields) see $tblrow
    }

    if { [design:hasTag $objid t_l] } {
        set PgAcVar(report,labeltext) [$Win(draft).c itemcget $objid -text]
    } elseif { [design:hasTag $objid t_o] } {
        set PgAcVar(report,formula) [$Win(draft).c itemcget $objid -text]
    } elseif { [design:hasTag $objid t_p] } {
        set PgAcVar(report,picture) [$Win(draft).c itemcget $objid -image]
    } else {
        # nothing
    }

}; # end proc ::Reports::design:selectField



#==========================================================
# The new portion of the Reports namespace follows.
#==========================================================



# ------------------------------------------------------------------------------
# Report
# ------------------------------------------------------------------------------
namespace eval ::Reports {
    variable Name
    variable RecSource
    variable Filter
    variable FilterOn
    variable OrderBy
    variable OrderByOn
    variable Caption
    variable RecLocks
    variable PageHead
    variable PageFoot
    variable DateGroup
    variable KeepTogether
    variable Width
    variable Pic
    variable PicType
    variable PicSizeMode
    variable PicAlign
    variable PicTiling
    variable PicPages
    variable MenuBar
    variable Toolbar
    variable ShortMenuBar
    variable GridX
    variable GridY
    variable LayoutPrint
    variable FastLaser
    variable HelpFile
    variable HelpContextID
    variable PaletteSrc
    variable Tag
    variable OnOpen
    variable OnClose
    variable OnActivate
    variable OnDeactivate
    variable OnNoData
    variable OnPage
    variable OnError
    variable HasModule
}

# ------------------------------------------------------------------------------
# Command ::Reports::create
#
# Values MSAccess uses for properties
# RecLocks - No Locks, All Records
# PageHead - All Pages, Not with Rpt Hdr, Not with Rpt Ftr, Not with Rpt Hdr/Ftr
# PageFoot - All Pages, Not with Rpt Hdr, Not with Rpt Ftr, Not with Rpt Hdr/Ftr
# DateGroup - Use System Settings, US Default
# KeepTogether - Per Column, Per Page
# Pic - The actual path to the picture
# PicType - Embedded, Link
# PicSizeMode - Clip, Stretch, Zoom
# PicPages - All Pages, First Page, No Pages
# OnOpen - [Event Procedure]- User written TCL script
# OnClose - [Event Procedure]- User written TCL script 
# OnActivate - [Event Procedure]- User written TCL script 
# OnDeactivate - [Event Procedure]- User written TCL script 
# OnNoData - [Event Procedure]- User written TCL script 
# OnPage - [Event Procedure]- User written TCL script 
# OnError - [Event Procedure]- User written TCL script 
# ------------------------------------------------------------------------------
proc ::Reports::create {path args} {

    variable $path
    upvar 0 $path data

    set Name {};      # Need to add code to increment Report 1, Report 2, ect...
    set RecSource {}; # Can be a query or table
    set Filter    {}
    set FilterOn 0
    set OrderBy {}
    set OrderByOn 0
    set Caption {}
    set RecLocks [intlmsg {No Locks}]
    set PageHead [intlmsg {All Pages}]
    set PageFoot [intlmsg {All Pages}]
    set DateGroup [intlmsg {Use System Settings}]
    set KeepTogether [intlmsg {Per Column}]
    set Width "5\""
    set Pic {(none)}
    set PicType [intlmsg {Embedded}]
    set PicSizeMode [intlmsg {Clip}]
    set PicAlign [intlmsg {Center}]
    set PicTiling 0
    set PicPages [intlmsg {All Pages}]
    set MenuBar {}
    set Toolbar {}
    set ShortMenuBar {}
    set GridX 24
    set GridY 24
    set LayoutPrint 1
    set FastLaser 1
    set HelpFile {}
    set HelpContextID 0
    set PaletteSrc {(Default)}
    set Tag {}
    set OnOpen {}
    set OnClose {}
    set OnActivate {}
    set OnDeactivate {}
    set OnNoData {}
    set OnPage {}
    set OnError {}
    set HasModule 0

}; # end proc ::Reports::create


# ------------------------------------------------------------------------------
# Report Header
# ------------------------------------------------------------------------------
namespace eval ::Reports::Head {
    variable Name
    variable ForceNewPg
    variable NewRowCol
    variable KeepTogether
    variable Visible
    variable Grow
    variable Shrink
    variable Heigh
    variable BackColor
    variable SpecEffect
    variable Tag
    variable OnFormat
    variable OnPrint
    variable OnRetreat
}

# ------------------------------------------------------------------------------
# Command ::Reports::Head::create
#
# Values MSAccess uses for properties
# ForceNewPg - None, Before Section, After Section, Before & After
# NewRowCol - None, Before Section, After Section, Before & After
# SpecEffect - Flat, Raised, Sunken
# OnFormat - [Event Procedure]- User written TCL script
# OnPrint - [Event Procedure]- User written TCL script 
# OnRetreat - [Event Procedure]- User written TCL script 
# ------------------------------------------------------------------------------
proc ::Reports::Head::create {path args} {

    variable $path
    upvar 0 $path data

    set Name [intlmsg {ReportHeader}]
    set ForceNewPg [intlmsg {None}]
    set NewRowCol [intlmsg {None}]
    set KeepTogether 1
    set Visible 1
    set Grow 0
    set Shrink 0
    set Height "0.25\""
    set BackColor \xFFFFFF; #White
    set SpecEffect [intlmsg {Flat}]
    set Tag {}
    set OnFormat {}
    set OnPrint {}
    set OnRetreat {}

}; # end proc ::Reports::Head::create {path args} {


# ------------------------------------------------------------------------------
# Page Header
# ------------------------------------------------------------------------------
namespace eval ::Reports::Page::Head {
    variable Name
    variable Visible
    variable Height
    variable BackColor
    variable SpecEffect
    variable Tag
    variable OnFormat
    variable OnPrint
}

# ------------------------------------------------------------------------------
# Command ::Reports::Page::Head::create
#
# Values MSAccess uses for properties
# SpecEffect - Flat, Raised, Sunken
# OnFormat - [Event Procedure]- User written TCL script
# OnPrint - [Event Procedure]- User written TCL script 
# ------------------------------------------------------------------------------
proc ::Reports::Page::Head::create {path args} {

    variable $path
    upvar 0 $path data

    set Name [intlmsg {PageHeaderSection}]
    set Visible 1
    set Height "0.25\""
    set BackColor \xFFFFFF; #White
    set SpecEffect [intlmsg {Flat}]
    set Tag {}
    set OnFormat {}
    set OnPrint {}

}; # end proc ::Reports::Page::Head::create


# ------------------------------------------------------------------------------
# Group Header
# ------------------------------------------------------------------------------
namespace eval ::Reports::Group::Head {
    variable Name
    variable ForceNewPg
    variable NewRowCol
    variable KeepTogether
    variable Visible
    variable Grow
    variable Shrink
    variable RepeatSection
    variable Height
    variable BackColor
    variable SpecEffect
    variable Tag
    variable OnFormat
    variable OnPrint
    variable OnRetreat
}

# ------------------------------------------------------------------------------
# Command ::Reports::Group::Head::create
#
# Values MSAccess uses for properties
# ForceNewPg - None, Before Section, After Section, Before & After
# NewRowCol - None, Before Section, After Section, Before & After
# SpecEffect - Flat, Raised, Sunken
# OnFormat - [Event Procedure]- User written TCL script
# OnPrint - [Event Procedure]- User written TCL script 
# OnRetreat - [Event Procedure]- User written TCL script 
# ------------------------------------------------------------------------------
proc ::Reports::Group::Head::create {path args} {

    variable $path
    upvar 0 $path data

    set Name [intlmsg {GroupHeader1} {};  # Need to add code to increment GroupHeader1, GroupHeader2, ect...
    set ForceNewPg [intlmsg {None}]
    set NewRowCol [intlmsg {None}]
    set KeepTogether 1
    set Visible 1
    set Grow 0
    set Shrink 0
    set RepeatSection 0
    set Height "0.25\""
    set BackColor \xFFFFFF; #White
    set SpecEffect [intlmsg {Flat}]
    set Tag {}
    set OnFormat {}
    set OnPrint {}
    set OnRetreat {}

}; # end proc ::Reports::Group::Head::create


# ------------------------------------------------------------------------------
# Report Detail
# ------------------------------------------------------------------------------
namespace eval ::Reports::Detail {
    variable Name
    variable ForceNewPg
    variable NewRowCol
    variable KeepTogether
    variable Visible
    variable Grow
    variable Shrink
    variable Height
    variable BackColor
    variable SpecEffect
    variable Tag
    variable OnFormat
    variable OnPrint
    variable OnRetreat
}

# ------------------------------------------------------------------------------
# Command ::Reports::Detail::create
#
# Values MSAccess uses for properties
# ForceNewPg - None, Before Section, After Section, Before & After
# NewRowCol - None, Before Section, After Section, Before & After
# SpecEffect - Flat, Raised, Sunken
# OnFormat - [Event Procedure]- User written TCL script
# OnPrint - [Event Procedure]- User written TCL script 
# OnRetreat - [Event Procedure]- User written TCL script 
# ------------------------------------------------------------------------------
proc ::Reports::Detail::create {path args} {

    variable $path
    upvar 0 $path data

    set Name [intlmsg {Detail}]
    set ForceNewPg [intlmsg {None}]
    set NewRowCol [intlmsg {None}]
    set KeepTogether 1
    set Visible 1
    set Grow 0
    set Shrink 0
    set Height "2\""
    set BackColor \xFFFFFF; #White
    set SpecEffect [intlmsg {Flat}]
    set Tag {}
    set OnFormat {}
    set OnPrint {}
    set OnRetreat {}

}; # end proc ::Reports::Detail::create


# ------------------------------------------------------------------------------
# Group Footer
# ------------------------------------------------------------------------------
namespace eval ::Reports::Group::Foot {
    variable Name
    variable ForceNewPg
    variable NewRowCol
    variable KeepTogether
    variable Visible
    variable Grow
    variable Shrink
    variable Height
    variable BackColor
    variable SpecEffect
    variable Tag
    variable OnFormat
    variable OnPrint
    variable OnRetreat
}

# ------------------------------------------------------------------------------
# Command ::Reports::Group::Foot::create
#
# Values MSAccess uses for properties
# ForceNewPg - None, Before Section, After Section, Before & After
# NewRowCol - None, Before Section, After Section, Before & After
# SpecEffect - Flat, Raised, Sunken
# OnFormat - [Event Procedure]- User written TCL script
# OnPrint - [Event Procedure]- User written TCL script 
# OnRetreat - [Event Procedure]- User written TCL script 
# ------------------------------------------------------------------------------
proc ::Reports::Group::Foot::create {path args} {

    variable $path
    upvar 0 $path data

    set Name [intlmsg {GroupFooter1}; # Need to add code to increment GroupFooter1, GroupFooter2, ect...
    set ForceNewPg [intlmsg {None}]
    set NewRowCol [intlmsg {None}]
    set KeepTogether 1
    set Visible 1
    set Grow 0
    set Shrink 0
    set Height "0.25\""
    set BackColor \xFFFFFF; #White
    set SpecEffect [intlmsg {Flat}]
    set Tag {}
    set OnFormat {}
    set OnPrint {}
    set OnRetreat {}

}; # end proc ::Reports::Group::Foot::create {path args} {


# ------------------------------------------------------------------------------
# Page Footer
# ------------------------------------------------------------------------------
namespace eval ::Reports::Page::Foot {
    variable Name
    variable Visible
    variable Hieght
    variable BackColor
    variable SpecEffect
    variable Tag
    variable OnFormat
    variable OnPrint
}

# ------------------------------------------------------------------------------
# Command ::Reports::Page::Foot::create
#
# Values MSAccess uses for properties
# SpecEffect - Flat, Raised, Sunken
# OnFormat - [Event Procedure]- User written TCL script
# OnPrint - [Event Procedure]- User written TCL script 
# OnRetreat - [Event Procedure]- User written TCL script 
# ------------------------------------------------------------------------------
proc ::Reports::Page::Foot::create {path args} {

    variable $path
    upvar 0 $path data

    set Name [intlmsg {PageFooterSection}]
    set Visible 1
    set Hieght "0.25\""
    set BackColor \xFFFFFF; #White
    set SpecEffect [intlmsg {Flat}]
    set Tag {}
    set OnFormat {}
    set OnPrint {}

}; # end proc ::Reports::Page::Foot::create


# ------------------------------------------------------------------------------
# Report Footer
# ------------------------------------------------------------------------------
namespace eval ::Reports::Foot {
    variable Name
    variable ForceNewPg
    variable NewRowCol
    variable KeepTogether
    variable Visible
    variable Grow
    variable Shrink
    variable Height
    variable BackColor
    variable SpecEffect
    variable Tag
    variable OnFormat
    variable OnPrint
    variable OnRetreat
}

# ------------------------------------------------------------------------------
# Command ::Reports::Foot::create
#
# Values MSAccess uses for properties
# ForceNewPg - None, Before Section, After Section, Before & After
# NewRowCol - None, Before Section, After Section, Before & After
# SpecEffect - Flat, Raised, Sunken
# OnFormat - [Event Procedure]- User written TCL script
# OnPrint - [Event Procedure]- User written TCL script 
# OnRetreat - [Event Procedure]- User written TCL script 
# ------------------------------------------------------------------------------
proc ::Reports::Foot::create {path args} {

    variable $path
    upvar 0 $path data

    set Name [intlmsg {ReportFooter}]
    set ForceNewPg [intlmsg {None}]
    set NewRowCol [intlmsg {None}]
    set KeepTogether 1
    set Visible 1
    set Grow 0
    set Shrink 0
    set Height "0.25\""
    set BackColor \xFFFFFF; #White
    set SpecEffect [intlmsg {Flat}]
    set Tag {}
    set OnFormat {}
    set OnPrint {}
    set OnRetreat {}

}; # end proc ::Reports::Foot::create


# ------------------------------------------------------------------------------
# Sub Report
# ------------------------------------------------------------------------------
namespace eval ::Reports::SubReport {
    variable Name
    variable SourceObject
    variable LinkChild
    variable LinkMaster
    variable Visible
    variable Grow
    variable Shrink
    variable Left
    variable Top
    variable Width
    variable Height
    variable SpecEffect
    variable BorderStyle
    variable BorderWidth
    variable BorderColor
    variable Tag

}

# ------------------------------------------------------------------------------
# Command ::Reports::SubReport::create
#
# Values MSAccess uses for properties
# SpecEffect - Flat, Raised, Sunken
# BorderStyle - Transparent, Solid, Dashes, Short Dashes, Dots, Sparse Dots,
#               Dash Dot, Dash Dot Dot
# BorderWidth - Hairline, 1 pt, 2 pt, 3 pt, 4 pt, 5 pt, 6 pt
# ------------------------------------------------------------------------------
proc ::Reports::SubReport::create {path args} {

    variable $path
    upvar 0 $path data

    set Name {};         # Name of the report, table or query
    set SourceObject {}; # Name of the report, table or query with "Report." prepended
    set LinkChild {};    # Name of field
    set LinkMaster {};   # Name of field
    set Visible 1
    set Grow 1
    set Shrink 0
    set Left {};         # Function of where you actually place widget
    set Top {};          # Function of where you actually place widget
    set Width {};        # Function of how you size widget
    set Height {};       # Function of how you size widget
    set SpecEffect [intlmsg {Flat}]
    set BorderStyle [intlmsg {Transparent}]
    set BorderWidth [intlmsg {Hairline}]
    set BorderColor 0;   #Black
    set Tag {}

}; # end proc ::Reports::SubReport::create



#==========================================================
# END REPORTS NAMESPACE
# BEGIN VISUAL TCL CODE
#==========================================================



# handmade but call it vTcl for continuity, someday use visualtcl again
proc vTclWindow.pgaw:ReportBuilder:draft {base} {

    global PgAcVar Win

    if {$base == ""} {
        set base .pgaw:ReportBuilder:draft
    }

    if {[winfo exists $base]} {
        wm deiconify $base; return
    }

    toplevel $base -class Toplevel
    wm focusmodel $base passive
    wm geometry $base 508x345+605+120
    wm maxsize $base 2560 2048
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base [intlmsg "Report draft"]

    # some helpful key bindings
    bind $base <Control-Key-w> [subst {destroy $base}]
    bind $base <Key-F1> "::Help::load reports"

    set ::Reports::Win(draft) $base

    canvas $base.c \
        -background #fffeff -borderwidth 2 -height 207 -highlightthickness 0 \
        -relief ridge -takefocus 1 -width 295 
    place $base.c \
        -x 0 -y 0 -width 508 -height 345 -anchor nw -bordermode ignore 
    bind $base.c <Button-1> {
        Reports::design:dragStart %W %x %y
    }
    bind $base.c <ButtonRelease-1> {
        Reports::design:dragStop %x %y
    }
    bind $base.c <Key-Delete> {
        Reports::design:deleteObject
    }
    bind $base.c <Motion> {
        Reports::design:dragMove %W %x %y
    }
    bind $base <Destroy> {
        if {[string match %W [winfo toplevel %W]]} {
            Reports::design:close
        }
    }
}


proc vTclWindow.pgaw:ReportBuilder:menu {base} {

    global PgAcVar Win

    if {$base == ""} {
        set base .pgaw:ReportBuilder:menu
    }

    if {[winfo exists $base]} {
        wm deiconify $base; return
    }

    toplevel $base -class Toplevel
    wm focusmodel $base passive
    wm geometry $base 600x600+20+20
    wm maxsize $base 2560 2048
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm deiconify $base
    wm title $base [intlmsg "Report menu"]

    # some helpful key bindings
    bind $base <Control-Key-w> [subst {destroy $base}]
    bind $base <Key-F1> "::Help::load reports"

    # if its a list box being destroyed let it go
    # we only want to close when the whole window closes
    bind $base <Destroy> {
        if {[string match %W [winfo toplevel %W]]} {
            Reports::design:close
        }
    }

    set ::Reports::Win(menu) $base

    # setup big frames
    frame $base.fname \
        -borderwidth 3 \
        -relief flat
    TitleFrame $base.fsourcefields \
        -text [intlmsg "Source fields"] \
        -borderwidth 3 \
        -relief groove
    TitleFrame $base.freportfields \
        -text [intlmsg "Report fields"] \
        -borderwidth 3 \
        -relief groove
    TitleFrame $base.fcomplexfields \
        -text [intlmsg "Complex fields"] \
        -borderwidth 3 \
        -relief groove

    frame $base.fbuttons \
        -borderwidth 3 \
        -relief flat
    TitleFrame $base.fpositions \
        -text [intlmsg "Region positions"] \
        -borderwidth 3 \
        -relief groove
    TitleFrame $base.fcurrent \
        -text [intlmsg "Current field"] \
        -borderwidth 3 \
        -relief groove

    grid $base.fname -column 0 -row 0 -columnspan 2 -rowspan 1 -sticky new
    grid $base.fsourcefields -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky news
    grid $base.freportfields -column 1 -row 1 -columnspan 1 -rowspan 1 -sticky news
    grid $base.fcomplexfields -column 0 -row 2 -columnspan 2 -rowspan 1 -sticky news
    grid $base.fbuttons -column 2 -row 0 -columnspan 1 -rowspan 1 -sticky ne
    grid $base.fpositions -column 2 -row 1 -columnspan 1 -rowspan 1 -sticky news
    grid $base.fcurrent -column 2 -row 2 -columnspan 1 -rowspan 1 -sticky news

    grid columnconfigure $base 1 -weight 2
    grid rowconfigure $base 1 -weight 2

    # report fields
    set base .pgaw:ReportBuilder:menu
    set base [$base.fcurrent getframe]
    set ::Reports::Win(frp) $base

    label $base.lselreg \
        -borderwidth 1 -relief raised \
        -text [intlmsg {Report region}] \
        -width 20
    label $base.lmsg \
        -relief groove -text [intlmsg {Report header}] \
        -textvariable PgAcVar(report,msg) 
    label $base.lseltype \
        -borderwidth 1 -relief raised \
        -text [intlmsg {Text type}]
    label $base.linfo \
        -relief groove -text [intlmsg {Database field}] \
        -textvariable PgAcVar(report,info) 

    label $base.lfont \
        -borderwidth 1 -relief raised \
        -text [intlmsg {Font}]
    Button $base.bfont \
        -helptext [intlmsg {Choose Font}] \
        -borderwidth 1 \
        -command Reports::design:setFont \
        -relief groove -text "Courier 14" \
        -width 5
    label $base.llal \
        -borderwidth 0 -text Align 
    Button $base.balign \
        -helptext [intlmsg {Choose Alignment}] \
        -borderwidth 0 -command Reports::design:toggleAlignMode \
        -relief groove -text right 

    grid $base.lselreg \
        -in $base -column 0 -row 0 -columnspan 4 -rowspan 1 -sticky new
    grid $base.lmsg \
        -in $base -column 0 -row 1 -columnspan 4 -rowspan 1 -sticky new
    grid $base.lseltype \
        -in $base -column 0 -row 2 -columnspan 4 -rowspan 1 -sticky new
    grid $base.linfo \
        -in $base -column 0 -row 3 -columnspan 4 -rowspan 1 -sticky new
    grid $base.lfont \
        -in $base -column 0 -row 4 -columnspan 4 -rowspan 1 -sticky new
    grid $base.bfont \
        -in $base -column 0 -row 5 -columnspan 4 -rowspan 1 -sticky new
    grid $base.llal \
        -in $base -column 0 -row 6 -columnspan 1 -rowspan 1 -sticky e
    grid $base.balign \
        -in $base -column 1 -row 6 -columnspan 3 -rowspan 1 -sticky ew


    set base .pgaw:ReportBuilder:menu
    set base [$base.fsourcefields getframe]
    set ::Reports::Win(srcfields) $base.lb
    scrollbar $base.sb \
        -command [subst {$base.lb yview}] \
        -orient vert
    listbox $base.lb \
        -width 10 \
        -yscrollcommand [subst {$base.sb set}]
    bind $base.lb <ButtonRelease-1> {
        Reports::design:addField
    }

    grid $base.lb \
        -in $base -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky news
    grid $base.sb \
        -in $base -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns
    grid columnconfigure $base 0 -weight 2
    grid rowconfigure $base 0 -weight 2

    set base .pgaw:ReportBuilder:menu
    set base [$base.freportfields getframe]
    scrollbar $base.sby \
        -command [subst {$base.tbl yview}] \
        -orient vert
    scrollbar $base.sbx \
        -command [subst {$base.tbl xview}] \
        -orient horizontal
    set tblhead [list \
        0 [intlmsg "id"] left \
        0 [intlmsg "x"] left \
        0 [intlmsg "y"] left \
        0 [intlmsg "type"] left \
        0 [intlmsg "value"] left]
    tablelist::addBWidgetSpinBox SpinBox
    set ::Reports::Win(usedfields) [tablelist::tablelist $base.tbl \
        -labelcommand tablelist::sortByColumn \
        -font PGfont2 \
        -labelfont PGfont \
        -stretch all \
        -columns $tblhead \
        -height 3 \
        -selectbackground #DDDDDD \
        -selectforeground #708090 \
        -editstartcommand ::Reports::design:editStart \
        -editendcommand ::Reports::design:editEnd \
        -xscrollcommand [subst {$base.sbx set}] \
        -yscrollcommand [subst {$base.sby set}]]
    $base.tbl columnconfigure 0 \
        -name id \
        -editable 0 \
        -sortmode integer
    $base.tbl columnconfigure 1 \
        -name x \
        -sortmode integer \
        -editable 1 \
        -editwindow SpinBox
    $base.tbl columnconfigure 2 \
        -name y \
        -sortmode integer \
        -editable 1 \
        -editwindow SpinBox
    $base.tbl columnconfigure 3 \
        -name type \
        -editable 0
    $base.tbl columnconfigure 4 \
        -name value \
        -editable 1

    bind [$base.tbl bodytag] <Button-1> {
        ::Reports::design:selectRegionFieldFromMenu %W %x %y
    }

    bind [$base.tbl bodytag] <Key-Delete> {
        ::Reports::design:deleteObject
    }

    grid $base.tbl \
        -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky news
    grid $base.sby \
        -column 1 -row 0 -columnspan 1 -rowspan 1 -sticky ns
    grid $base.sbx \
        -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky ew
    grid columnconfigure $base 0 -weight 2
    grid rowconfigure $base 0 -weight 2

    # control buttons box
    set base .pgaw:ReportBuilder:menu
    set base $base.fbuttons
    set ::Reports::Win(fbtn) $base
    ButtonBox $base.bbox \
        -orient horizontal \
        -homogeneous 1 \
        -spacing 2
    $base.bbox add \
        -relief link \
        -image ::icon::filesave-22 \
        -borderwidth 1 \
        -command {::Reports::design:save 0} \
        -helptext [intlmsg {Save}]
    $base.bbox add \
        -relief link \
        -image ::icon::filesaveas-22 \
        -borderwidth 1 \
        -command {::Reports::design:save 1} \
        -helptext [intlmsg {Save As}]
    $base.bbox add \
        -relief link \
        -image ::icon::imagegallery-22 \
        -borderwidth 1 \
        -command {::Reports::design:preview} \
        -helptext [intlmsg {Preview}]
    $base.bbox add \
        -relief link \
        -image ::icon::editdelete-22 \
        -borderwidth 1 \
        -command {::Reports::design:deleteAllObjects} \
        -helptext [intlmsg {Delete all}]
    $base.bbox add \
        -relief link \
        -image ::icon::help-22 \
        -borderwidth 1 \
        -command {::Help::load reports} \
        -helptext [intlmsg {Help}]
    $base.bbox add \
        -relief link \
        -image ::icon::exit-22 \
        -borderwidth 1 \
        -command {::Reports::design:close} \
        -helptext [intlmsg {Close}]


    # report size
    label $base.lrsize \
        -borderwidth 1 -relief raised \
        -text [intlmsg {Report size}]
    SpinBox $base.erw \
        -relief groove \
        -textvariable PgAcVar(report,rw) \
        -justify right \
        -range {0 2560 1} \
        -modifycmd {Reports::design:changeDraftCoords} \
        -text 345 \
        -width 4
    label $base.lrwbyh \
        -borderwidth 0 \
        -text " x "
    SpinBox $base.erh \
        -relief groove \
        -textvariable PgAcVar(report,rh) \
        -justify right \
        -range {0 2048 1} \
        -modifycmd {Reports::design:changeDraftCoords} \
        -text 508 \
        -width 4
    # page size
    label $base.lpsize \
        -borderwidth 1 -relief raised \
        -text [intlmsg {Page size}]
    SpinBox $base.epw \
        -relief groove \
        -textvariable PgAcVar(report,pw) \
        -justify right \
        -range {0 2560 1} \
        -text 345 \
        -width 4
    label $base.lpwbyh \
        -borderwidth 0 \
        -text " x "
    SpinBox $base.eph \
        -relief groove \
        -textvariable PgAcVar(report,ph) \
        -justify right \
        -range {0 2048 1} \
        -text 508 \
        -width 4

    grid $base.bbox \
        -in $base -column 0 -row 0 -columnspan 3 -rowspan 1 -sticky ne
    grid $base.lrsize \
        -in $base -column 0 -row 1 -columnspan 3 -rowspan 1 -sticky we
    grid $base.erw \
        -in $base -column 0 -row 2 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.lrwbyh \
        -in $base -column 1 -row 2 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.erh \
        -in $base -column 2 -row 2 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.lpsize \
        -in $base -column 0 -row 3 -columnspan 3 -rowspan 1 -sticky we
    grid $base.epw \
        -in $base -column 0 -row 4 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.lpwbyh \
        -in $base -column 1 -row 4 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.eph \
        -in $base -column 2 -row 4 -columnspan 1 -rowspan 1 -sticky nsew

    # report name and source frame
    set base .pgaw:ReportBuilder:menu
    set base $base.fname
    set ::Reports::Win(fns) $base
    label $base.lrn \
        -borderwidth 0 -text [intlmsg {Report name}]
    entry $base.ern \
        -background #fefefe -borderwidth 1 -highlightthickness 0 \
        -textvariable PgAcVar(report,reportname) \
        -width 35
    bind $base.ern <Key-F5> {
        loadReport
    }
    label $base.lrs \
        -borderwidth 0 -text [intlmsg {Report source}]
    ComboBox $base.cbrs \
        -values [concat [::Database::getPrefObjList Tables] \
            [::Database::getPrefObjList Views]] \
        -editable true \
        -textvariable PgAcVar(report,tablename) \
        -modifycmd {::Reports::design:getSourceFieldsForFieldBox}

    label $base.lrfilter \
        -borderwidth 0 -text [intlmsg "Report filter"]
    entry $base.erfilter \
        -background #fefefe -borderwidth 1 -highlightthickness 0 \
        -textvariable PgAcVar(report,filter) \
        -width 35
    
    label $base.lrorderby \
        -borderwidth 0 -text [intlmsg "Report order by"]
    entry $base.erorderby \
        -background #fefefe -borderwidth 1 -highlightthickness 0 \
        -textvariable PgAcVar(report,orderby) \
        -width 35
    
    grid $base.lrn \
         -column 0 -row 0 -columnspan 1 -rowspan 1 -sticky e
    grid $base.ern \
         -column 1 -row 0 -columnspan 2 -rowspan 1 -sticky ew
    grid $base.lrs \
         -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky e
    grid $base.cbrs \
         -column 1 -row 1 -columnspan 2 -rowspan 1 -sticky ew
    grid $base.lrfilter \
         -column 0 -row 2 -columnspan 1 -rowspan 1 -sticky e
    grid $base.erfilter \
         -column 1 -row 2 -columnspan 2 -rowspan 1 -sticky ew
    grid $base.lrorderby \
         -column 0 -row 3 -columnspan 1 -rowspan 1 -sticky e
    grid $base.erorderby \
         -column 1 -row 3 -columnspan 2 -rowspan 1 -sticky ew
    grid columnconfigure $base 1 -weight 2

    # add label, formula, picture
    set base .pgaw:ReportBuilder:menu
    set base [$base.fcomplexfields getframe]
    set ::Reports::Win(fadd) $base
    entry $base.elab \
        -background #fefefe -borderwidth 1 -highlightthickness 0 \
        -textvariable PgAcVar(report,labeltext)
    bind $base.elab <Key-Return> {
        Reports::design:updateObject
    }
    button $base.badl \
        -borderwidth 1 -command Reports::design:addLabel \
        -text [intlmsg {Add label}]
    button $base.bupl \
        -borderwidth 1 -command Reports::design:updateObject \
        -text [intlmsg {Update label}]
    button $base.beditlabel \
        -borderwidth 1 -command "
            Window show .pgaw:ReportBuilder:commands
            set PgAcVar(report,commandType) labeltext
            .pgaw:ReportBuilder:commands.f.txt delete 1.0 end
            .pgaw:ReportBuilder:commands.f.txt insert end \$PgAcVar(report,labeltext)" \
        -text [intlmsg {Edit}]
    ComboBox $base.ef \
        -textvariable PgAcVar(report,formula) \
        -editable 1 \
        -values [list Function::some_function(some_field) PGAImage::some_image_name ImageData::some_field_with_image_data Barcode::some_field]
    bind $base.ef <Key-Return> {
        Reports::design:updateObject
    }
    button $base.baf \
        -borderwidth 1 -command Reports::design:addFormula \
        -text [intlmsg {Add formula}]
    button $base.buf \
        -borderwidth 1 -command Reports::design:updateObject \
        -text [intlmsg {Update formula}]
    button $base.beditformula \
        -borderwidth 1 -command "
            Window show .pgaw:ReportBuilder:commands
            set PgAcVar(report,commandType) formula
            .pgaw:ReportBuilder:commands.f.txt delete 1.0 end
            .pgaw:ReportBuilder:commands.f.txt insert end \$PgAcVar(report,formula)
            Syntax::highlight .pgaw:ReportBuilder:commands.f.txt tcl" \
        -text [intlmsg {Edit}]
    entry $base.ep \
        -background #fefefe -borderwidth 1 -highlightthickness 0 \
        -textvariable PgAcVar(report,picture) 
    button $base.bap \
        -borderwidth 1 -command Reports::design:addPicture \
        -text [intlmsg {Add picture}]
    button $base.bbrowsepic \
        -borderwidth 1 -command Reports::design:browsePicture \
        -text [intlmsg {Browse}]
    
    grid $base.elab \
        -in $base -column 0 -row 0 -columnspan 3 -rowspan 1 -sticky nsew
    grid $base.beditlabel \
        -in $base -column 3 -row 0 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.badl \
        -in $base -column 0 -row 1 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.bupl \
        -in $base -column 1 -row 1 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.ef \
        -in $base -column 0 -row 2 -columnspan 3 -rowspan 1 -sticky nsew
    grid $base.beditformula \
        -in $base -column 3 -row 2 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.baf \
        -in $base -column 0 -row 3 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.buf \
        -in $base -column 1 -row 3 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.ep \
        -in $base -column 0 -row 4 -columnspan 3 -rowspan 1 -sticky nsew
    grid $base.bbrowsepic \
        -in $base -column 3 -row 4 -columnspan 1 -rowspan 1 -sticky nsew
    grid $base.bap \
        -in $base -column 0 -row 5 -columnspan 1 -rowspan 1 -sticky nsew
    grid columnconfigure $base 2 -weight 2

    # regions and fields
    set base .pgaw:ReportBuilder:menu
    set base [$base.fpositions getframe]
    set ::Reports::Win(frgf) $base
    set row 0
    foreach reg {rpthdr pghdr detail pgfoo rptfoo} \
        regtit {"Report Header" "Page Header" "Detail" "Page Footer" "Report Footer"} {
        label $base.lrg$reg \
            -borderwidth 0 -text [intlmsg $regtit] -justify right
        SpinBox $base.spinrg$reg \
            -textvariable PgAcVar(report,y_$reg) \
            -justify right \
            -range {0 9999 1} \
            -modifycmd ::Reports::design:drawReportAreas \
            -width 4
        grid $base.lrg$reg \
            -in $base -column 0 -row $row -columnspan 1 -rowspan 1 \
            -sticky ne
        grid $base.spinrg$reg \
            -in $base -column 1 -row $row -columnspan 1 -rowspan 1 \
            -sticky nw
        incr row
    }

}


proc vTclWindow.pgaw:ReportBuilder:commands {base} {

    global PgAcVar Win

    if {$base==""} {
        set base .pgaw:ReportBuilder:commands
    }

    if {[winfo exists $base]} {
        wm deiconify $base; return
    }

    toplevel $base -class Toplevel
    wm focusmodel $base passive
    wm geometry $base 640x480+120+100
    wm maxsize $base 785 570
    wm minsize $base 1 19
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm title $base [intlmsg "text"]

    # some helpful key bindings
    bind $base <Control-Key-w> [subst {destroy $base}]
    bind $base <Key-F1> "::Help::load reports"

    frame $base.f \
        -borderwidth 2 -height 75 -relief groove -width 125 
    scrollbar $base.f.sb \
        -borderwidth 1 -command {.pgaw:ReportBuilder:commands.f.txt yview} \
        -orient vert -width 12 
    text $base.f.txt \
        -font "$PgAcVar(pref,font_fix)" -height 1 \
        -tabs {20 40 60 80 100 120 140 160 180 200} \
        -width 200 -yscrollcommand {.pgaw:ReportBuilder:commands.f.sb set} \
        -background #ffffff
    frame $base.fb \
        -height 75 -width 125 
    button $base.fb.b1 \
        -borderwidth 1 \
        -command {
            set bad_tcl "yes"
            if {![info complete [.pgaw:ReportBuilder:commands.f.txt \
                get 1.0 "end - 1 chars"]]} {
                set bad_tcl [tk_messageBox -title [intlmsg Warning] \
                    -parent .pgaw:ReportBuilder:commands -type yesno \
                    -message [intlmsg "There appears to be invalid Tcl code.  Are you sure you want to save it?"]]
            } 
            if {$bad_tcl=="yes"} {
                set PgAcVar(report,$PgAcVar(report,commandType)) [.pgaw:ReportBuilder:commands.f.txt get 1.0 "end - 1 chars"]
                Window hide .pgaw:ReportBuilder:commands
                Reports::design:updateObject    
            }
        } -text [intlmsg Save] -width 5 
    button $base.fb.b2 \
        -borderwidth 1 -command {Window hide .pgaw:ReportBuilder:commands} \
        -text [intlmsg Cancel]
    # add Ctrl-x|c|v for cut, copy, paste
    bind $base.f.txt <Control-Key-x> {
        set PgAcVar(shared,curseltext) [%W get sel.first sel.last]    
        %W delete sel.first sel.last
    }
    bind $base.f.txt <Control-Key-c> {
        set PgAcVar(shared,curseltext) [%W get sel.first sel.last]    
    }
    bind $base.f.txt <Control-Key-v> {
        if {[info exists PgAcVar(shared,curseltext)]} {
            catch {%W delete sel.first sel.last}
            %W insert insert $PgAcVar(shared,curseltext)
            %W see current
        }
    }
    pack $base.f \
        -in .pgaw:ReportBuilder:commands -anchor center -expand 1 \
        -fill both -side top 
    pack $base.f.sb \
        -in .pgaw:ReportBuilder:commands.f -anchor e -expand 1 \
        -fill y -side right 
    pack $base.f.txt \
        -in .pgaw:ReportBuilder:commands.f -anchor center -expand 1 \
        -fill both -side top 
    pack $base.fb \
        -in .pgaw:ReportBuilder:commands -anchor center -expand 0 \
        -fill none -side top 
    pack $base.fb.b1 \
        -in .pgaw:ReportBuilder:commands.fb -anchor center -expand 0 \
        -fill none -side left 
    pack $base.fb.b2 \
        -in .pgaw:ReportBuilder:commands.fb -anchor center -expand 0 \
        -fill none -side top 
}


proc vTclWindow.pgaw:ReportPreview {base} {

    global PgAcVar Win

    if {$base == ""} {
        set base .pgaw:ReportPreview
    }

    if {[winfo exists $base]} {
        wm deiconify $base; return
    }

    toplevel $base -class Toplevel
    wm focusmodel $base passive
#    wm geometry $base 495x500+230+50
    wm maxsize $base 2560 2048
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm title $base "Report preview"

    # some helpful key bindings
    bind $base <Control-Key-w> [subst {destroy $base}]
    bind $base <Control-Key-p> ::Reports::design:print
    bind $base <Key-F1> "::Help::load reports"

    set ::Reports::Win(preview) $base

    frame $base.fr \
        -borderwidth 1 -height 75 -relief groove -width 125 
    canvas $base.fr.c \
        -background #fcfefe -borderwidth 2 -height 207 -relief ridge \
        -scrollregion {0 0 2560 2048} -width 295 \
        -yscrollcommand [subst {$base.fr.sb set}] 
    scrollbar $base.fr.sb \
        -borderwidth 1 -command [subst {$base.fr.c yview}] \
        -highlightthickness 0 \
        -orient vert -width 12 
    frame $base.fp \
        -borderwidth 1 -height 75 -width 125 

    Button $base.fp.bclose \
        -helptext [intlmsg {Close}] \
        -borderwidth 1 -command {
            if {$PgAcVar(report,justpreview)} {
                Window destroy .pgaw:ReportBuilder:menu
                Window destroy .pgaw:ReportBuilder:draft
            }
            Window destroy .pgaw:ReportPreview
        } -image ::icon::exit-22

    Button $base.fp.bprint \
        -helptext [intlmsg {Print}] \
        -borderwidth 1 -command Reports::design:print \
        -image ::icon::fileprint-22

    label $base.fp.ltexttotal -text "pages "
    label $base.fp.ltotal -textvariable PgAcVar(report,total_page)

    Button $base.fp.bfirst \
        -helptext [intlmsg {First Page}] \
        -image ::icon::2leftarrow-22
    bind $base.fp.bfirst <Button-1> { 
        if {$PgAcVar(report,curr_page)>1} {
            set PgAcVar(report,curr_page) 1
        }
        Reports::design:previewPage
    }

    Button $base.fp.bprev \
        -helptext [intlmsg {Previous Page}] \
        -image ::icon::1leftarrow-22
    bind $base.fp.bprev <Button-1> { 
        if {$PgAcVar(report,curr_page)>1} {
            set PgAcVar(report,curr_page) [expr {$PgAcVar(report,curr_page)-1}]
        }
        Reports::design:previewPage
    }

    Button $base.fp.bnext \
        -helptext [intlmsg {Next Page}] \
        -image ::icon::1rightarrow-22
    bind $base.fp.bnext <Button-1> { 
        if {$PgAcVar(report,curr_page)<$PgAcVar(report,last_page)} {
            set PgAcVar(report,curr_page) [expr {$PgAcVar(report,curr_page)+1}]
        }
        Reports::design:previewPage
    }

    Button $base.fp.blast \
        -helptext [intlmsg {Last Page}] \
        -image ::icon::2rightarrow-22
    bind $base.fp.blast <Button-1> { 
        if {$PgAcVar(report,curr_page)<$PgAcVar(report,last_page)} {
            set PgAcVar(report,curr_page) $PgAcVar(report,last_page)
        }
        Reports::design:previewPage
    }

    Entry $base.fp.estart \
        -helptext [intlmsg {Current Page to Start Printing}] \
        -width 5 \
        -textvariable PgAcVar(report,curr_page)
    bind $base.fp.estart <Key-Return> { 
        Reports::design:previewPage
    }

    label $base.fp.lthru -text -

    Entry $base.fp.estop \
        -helptext [intlmsg {Last Page to Stop Printing}] \
        -width 5 \
        -textvariable PgAcVar(report,last_page)
    bind $base.fp.estop <Key-Return> { 
        Reports::design:previewPage
    }

    pack $base.fr \
        -in .pgaw:ReportPreview -anchor center -expand 1 \
        -fill both -side top 
    pack $base.fr.c \
        -in .pgaw:ReportPreview.fr -anchor center -expand 1 \
        -fill both -side left 
    pack $base.fr.sb \
        -in .pgaw:ReportPreview.fr -anchor center -expand 0 \
        -fill y -side right 
    pack $base.fp \
        -in .pgaw:ReportPreview -anchor center -expand 0 \
        -fill none -side bottom
    pack $base.fp.ltotal \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.ltexttotal \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.bfirst \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.bprev \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.bnext \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.blast \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.estart \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.lthru \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.estop \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.bprint \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left
    pack $base.fp.bclose \
        -in .pgaw:ReportPreview.fp -expand 0 -fill none -side left

    if {[info exists PgAcVar(PGACCESS_NICEREPORTPREVIEW)]} {
        pack propagate $base 0
        update idletasks
        wm geometry $base +0+0
        wm minsize $base [expr {round(.80*[winfo screenwidth $base])}] \
                         [expr {round(.80*[winfo screenheight $base])}]
        wm maxsize $base [expr {round(.80*[winfo screenwidth $base])}] \
                         [expr {round(.80*[winfo screenheight $base])}]
    }

}
