#==========================================================
# Rules --
#
#   handling of PostgreSQL rules
#==========================================================
#
namespace eval Rules {}


#----------------------------------------------------------
# ::Rules::new --
#
#   sets up to create a new Rule
#
# Arguments:
#   none
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Rules::new {} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Rules::new


#----------------------------------------------------------
# ::Rules::open --
#
#   passes work to design proc for opening a Rule
#
# Arguments:
#   rule_    name of Rule to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Rules::open {rule_} {

    ::Rules::design $rule_

}; # end proc ::Rules::open


#----------------------------------------------------------
# ::Rules::design --
#
#   opens window in design mode for selected Rule
#
# Arguments:
#   rule_    name of Rule to open in design mode
#
# Returns:
#   none
#----------------------------------------------------------
#
proc ::Rules::design {rule_} {

    showError [intlmsg "Not yet implemented."]

}; # end proc ::Rules::new


#----------------------------------------------------------
# ::Rules::introspect --
#
#   Given a rulename, returns the SQL needed to recreate it
#
# Arguments:
#   rulename_   name of a rule to introspect
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to make this rule
#----------------------------------------------------------
#
proc ::Rules::introspect {rulename_ {dbh_ ""}} {

    set insql [::Rules::clone $rulename_ $rulename_ $dbh_]

    return $insql

}; # end proc ::Rules::introspect


#----------------------------------------------------------
# ::Rules::clone --
#
#   Like introspect, only changes the rulename
#
# Arguments:
#   srcrule_    the original rule
#   destrule_   the clone rule
#   dbh_        an optional database handle
#
# Returns:
#   insql       the CREATE statement to clone this rule
#
# Remarks:
#   eventually pull the string map in favor of generating
#   here completely the SQL needed to create the rule
#----------------------------------------------------------
#
proc ::Rules::clone {srcrule_ destrule_ {dbh_ ""}} {

    global CurrentDB

    if {[string match "" $dbh_]} {
        set dbh_ $CurrentDB
    }

    set V [::Database::getPgVersion $dbh_]

    set insql ""

    set srcrule_ [string map {\" {}} $srcrule_]
    set destrule_ [string map {\" {}} $destrule_]

    # we have to butcher this because the table name is
    # listed in the main window, like it is for triggers
    set srctbl [join [lrange [split $srcrule_ .] 0 end-1] .]
    set srcrul [lindex [split $srcrule_ .] end]
    set dsttbl [join [lrange [split $destrule_ .] 0 end-1] .]
    set dstrul [lindex [split $destrule_ .] end]

    # dont allow empty rule table name
    if {[string length $dsttbl]==0} {
        set dsttbl $srctbl
    }

    set sql ""

    if {$V<7.3} {
        set sql "SELECT definition
                   FROM pg_rules
                  WHERE rulename='$srcrul'"
    } else {
        set nsp [lindex [split $srctbl .] 0]
        set srctbl [lindex [split $srctbl .] 1]
        set sql "SELECT definition
                   FROM pg_rules
                  WHERE rulename='$srcrul'
                    AND schemaname='$nsp'"
    }

    wpg_select $dbh_ $sql rec {
        set insql [string map [list $srctbl $dsttbl $srcrul $dstrul] $rec(definition)]
    }

    return $insql

}; # end proc ::Rules::clone


