#----------------------------------------------------------
# splash.tcl
#
#    The splash screen on startup
#
#----------------------------------------------------------

#----------------------------------------------------------
# vTclWindow.pgaw:Splash --
#
#    Displays the Splash window
#
# Arguments:
#    base: the name of the window
#
# Results:
#    none returned.
#----------------------------------------------------------
#
proc vTclWindow.pgaw:Splash {base} {
	global PgAcVar Bar
	set Bar(length) 0
	set Bar(max) 225
	
	if {$base == ""} {
		set base .pgaw:Splash
	}
	if {[winfo exists $base]} {
		wm deiconify $base; return
	}
	toplevel $base \
	-background white \
	-relief ridge \
	-bd 4
	
	wm geometry $base 270x280
	wm resizable $base 0 0
	wm overrideredirect $base 1
	#don't allow people to close this window
	wm protocol $base WM_DELETE_WINDOW {}
	
	set left [expr {[winfo screenwidth $base] / 2 - 270/2}]
	set top [expr {[winfo screenheight $base] / 2 - 280 / 2}]
	wm geometry $base +$left+$top
	wm title $base PGAccess
	
	image create photo $base.logo \
	-file [file join $PgAcVar(PGACCESS_HOME) images logo.gif]
	label $base.logocont \
	-highlightthickness 0 \
	-bd 0 \
	-image $base.logo
	
	pack $base.logocont  \
	-padx 10 \
	-pady 10
	
	frame $base.frame
	pack $base.frame
	
	#label $base.message1 -text "pgaccess.org"
	label $base.message1 \
	-text "Version: [get_version]" \
	-foreground #888888 \
	-background #ffffff
	
	pack $base.message1 \
	-fill x \
	-expand 1 \
	-pady 6
	
	label $base.message2 \
	-background white
	pack $base.message2 -fill x -expand 1
	
	canvas $base.bar \
	-width $Bar(max) \
	-height 20 \
	-relief sunken \
	-bd 2 \
	-highlightthickness 0
	
	pack $base.bar \
	-padx 4 \
	-pady 10
	
	set clrcnt 0
	#TUBE: #405f7d #466582 #4d6b87 #54718c #5b7791 #617d97 #68839c #6f89a1 #768fa6 #7d95ab #6b839a #627b93 #5a738c #516b85 #48637e #405c77 #375470 #2f4c69 #264462 #1e3d5b
	
	foreach C {#8098ae #8098ae #4d6b87 #54718c #5b7791 #617d97 #68839c #6f89a1 #768fa6 #7d95ab #839bb1 #8aa1b6 #91a7bb #98adc0 #9fb3c5 #a5b9cb #acbfd0 #b3c5d5 #244361 #244361} {
		
		set y [expr {$clrcnt +2}]
		$base.bar create line -224 $y 1 $y \
		-fill $C \
		-tags [list $C bar]
		incr clrcnt
		
	}
	
	update idletasks
	
}

#----------------------------------------------------------
# slash_puts --
#
#    Displays string on the Splash window
#
# Arguments:
#    data: the string
#    mainevent: the level of the message
#
# Results:
#    none returned.
#----------------------------------------------------------
#
proc splash_puts {data {mainevent 0}} {
	if { [winfo exists .pgaw:Splash] && !$PgAcVar(PGACCESS_NOSPLASH) } {
		if {$mainevent} {
			.pgaw:Splash.message1 config -text $data
			.pgaw:Splash.message2 config -text ""
		} else {
			.pgaw:Splash.message2 config -text $data
		}
		update idletasks
    }
}

#----------------------------------------------------------
# splash_percent --
#
#    Handles the percent bar on the Splash scree
#
# Arguments:
#    percent: the filled percent of the percent bar
#
# Results:
#    none returned.
#----------------------------------------------------------
#
proc splash_percent {percent} {
	global PgAcVar Bar
	# short circuit if splash is disabled
	if { [winfo exists .pgaw:Splash] && !$PgAcVar(PGACCESS_NOSPLASH) } {
		#.pgaw:Splash.bar config -width [expr {($percent / 100.0) *
		#([winfo height .pgaw:Splash] - 30)}]
		set wid [expr {($percent * $Bar(max))/100}]
		.pgaw:Splash.bar move bar [expr {$wid - $Bar(length)}] 0
		set Bar(length) $wid
		update idletasks
	}
}
