#----------------------------------------------------------
# window.tcl
#
#    Window handling functions
#
#----------------------------------------------------------

image create bitmap dnarw -data  {
    #define down_arrow_width 15
    #define down_arrow_height 15
    static char down_arrow_bits[] = {
        0x00,0x80,0x00,0x80,0x00,0x80,0x00,0x80,
        0x00,0x80,0xf8,0x8f,0xf0,0x87,0xe0,0x83,
        0xc0,0x81,0x80,0x80,0x00,0x80,0x00,0x80,
        0x00,0x80,0x00,0x80,0x00,0x80
    }
}

#----------------------------------------------------------
# moreText --
#
#   Allows an elipsis (...) button, for example, to open a
#   new window with some text that possibly couldn't be
#   displayed all the way in a user GUI.
#
# Arguments:
#   text_       text to display in the window
#   editable_   1 if possible to save text, 0 if not
#   var_        variable name to save output text as
#
# Results:
#   none
#----------------------------------------------------------
#
proc moreText {text_ editable_ var_} {
    
    global PgAcVar
    
    # allow multiple moreText windows
    if { ![info exists PgAcVar(moreText,wincnt)] } {
        set PgAcVar(moreText,wincnt) 0
    }
    set wn [incr PgAcVar(moreText,wincnt)]

    set PgAcVar(moreText,$wn,save) 0
    set PgAcVar(moreText,$wn,text) $text_
    set PgAcVar(moreText,$wn,var) $var_

    set base .pgaw:moreText:$wn
    
    if {[winfo exists $base]} {
        wm deiconify $base
    } else {
        toplevel $base -class Toplevel
        wm focusmodel $base passive
        wm geometry $base 600x500+100+100
        wm overrideredirect $base 0
        wm resizable $base 1 1
        wm deiconify $base
        wm title $base [intlmsg "More Text"]
    }
    
    bind $base <Control-Key-w> [subst {
        destroy $base
    }]
    if {$editable_} {
        bind $base <Control-Key-s> [subst {
            global PgAcVar
            set PgAcVar(moreText,$wn,save) 1
            set PgAcVar(moreText,$wn,text) \
                \[string trim \[$base.txt get 1.0 end\]\]
            destroy $base
        }]
    }
    bind $base <Destroy> [subst {
        global PgAcVar
        if {\$PgAcVar(moreText,$wn,save)} {
            set \$PgAcVar(moreText,$wn,var) \$PgAcVar(moreText,$wn,text)
        }
    }]

    text $base.txt \
        -undo 1 \
        -yscrollcommand "$base.yscroll set" \
        -xscrollcommand "$base.xscroll set"
    scrollbar $base.yscroll \
        -command "$base.txt yview"
    scrollbar $base.xscroll \
        -orient horiz \
        -command "$base.txt xview"

    # default text paste is Control-Key-y
    bind $base.txt <Control-Key-v> {
        tk_textPaste %W
        break
    }
    # default select all is Control-Key-/
    bind $base.txt <Control-Key-a> {
        %W tag add sel 0.0 end
        break
    }

    Button $base.save \
    -borderwidth 1 \
    -image ::icon::filesave-22 \
    -helptext [intlmsg "Save and Exit (Ctrl-S)"] \
    -relief link \
    -command [subst {
        global PgAcVar
        set PgAcVar(moreText,$wn,save) 1
        set PgAcVar(moreText,$wn,text) \
            \[string trim \[$base.txt get 1.0 end\]\]
        destroy $base
    }]
    Button $base.exit \
    -borderwidth 1 \
    -image ::icon::exit-22 \
    -helptext [intlmsg "Cancel (Ctrl-W)"] \
    -relief link \
    -command [subst {
        Window destroy $base
    }]
    
    grid $base.txt \
    -row 0 \
    -column 0 \
    -columnspan 2 \
    -sticky news
    grid $base.yscroll \
    -row 0 \
    -column 2 \
    -sticky swn
    grid $base.xscroll \
    -row 1 \
    -column 0 \
    -columnspan 2 \
    -sticky wen
    if {$editable_} {
        grid $base.save \
        -row 2 \
        -column 0 \
        -sticky e
    }
    grid $base.exit \
    -row 2 \
    -column 1 \
    -sticky w
    grid columnconfigure $base 0 \
    -weight 10
    grid columnconfigure $base 1 \
    -weight 10
    grid rowconfigure $base 0 \
    -weight 10
    
    Window show $base
    $base.txt delete 1.0 end
    $base.txt insert end $text_
    
}; # end proc moreText


#----------------------------------------------------------
# create_drop_down --
#
#    ???
#
# Arguments:
#    base
#    x
#    y
#    w
#
# Results:
#    none returned.
#----------------------------------------------------------
#
proc create_drop_down {base x y w} {
    global PgAcVar
    if {[winfo exists $base.ddf]} return;
    frame $base.ddf -borderwidth 1 -height 75 -relief raised -width 55
    listbox $base.ddf.lb \
    -font $PgAcVar(pref,font_normal) \
    -highlightthickness 0 \
    -yscrollcommand [list $base.ddf.sb set]
    
    scrollbar $base.ddf.sb \
    -borderwidth 1 \
    -command [list $base.ddf.lb yview] \
    -orient vert
    
    place $base.ddf \
    -x $x \
    -y $y \
    -width $w \
    -height 185 \
    -anchor nw \
    -bordermode ignore
    
    place $base.ddf.lb \
    -x 1 \
    -y 1 \
    -width [expr {$w-18}] \
    -height 182 \
    -anchor nw \
    -bordermode ignore
    
    place $base.ddf.sb \
    -x [expr {$w-15}] \
    -y 1 \
    -width 14 \
    -height 183 \
    -anchor nw \
    -bordermode ignore
    
    return
}

#----------------------------------------------------------
# wpg_exec --
#
#    Executes an SQL command and displays the command in the SQL window
#
# Arguments:
#    db: database
#    cmd: sql command
#
# Results:
#    the result of the exec command
#----------------------------------------------------------
#
proc wpg_exec {db cmd} {
    global PgAcVar
    set PgAcVar(pgsql,cmd) "never executed"
    set PgAcVar(pgsql,status) "no status yet"
    set PgAcVar(pgsql,errmsg) "no error message yet"
    if {[catch {
        Mainlib::sqlw_display "-----BEGIN[string repeat "-" 40]"
        Mainlib::sqlw_display $cmd
        set PgAcVar(pgsql,cmd) $cmd
        set PgAcVar(pgsql,res) [pg_exec $db $cmd]
        set PgAcVar(pgsql,status) [pg_result $PgAcVar(pgsql,res) -status]
        set PgAcVar(pgsql,errmsg) [pg_result $PgAcVar(pgsql,res) -error]
    } tclerrmsg]} {
        showError [format [intlmsg "Tcl error executing pg_exec %s\n\n%s"] $cmd $tclerrmsg]
        return 0
    }
    Mainlib::sqlw_display $PgAcVar(pgsql,res)
    Mainlib::sqlw_display "-----END[string repeat "-" 40]"
    return $PgAcVar(pgsql,res)
}

#----------------------------------------------------------
# wpg_select --
#
#    executes a select command and displays the command in the SQL window
#
# Arguments:
#    args: the arguments of the select commands
#
# Results:
#    none returned.
#----------------------------------------------------------
#
proc wpg_select {args} {
    Mainlib::sqlw_display "-----BEGIN[string repeat "-" 40]"
    Mainlib::sqlw_display "[lindex $args 1]"
    Mainlib::sqlw_display "-----END[string repeat "-" 40]"
    uplevel pg_select $args
}

#----------------------------------------------------------
# set_cursor --
#
#    Sets the mouse cursor to the required shape
#
# Arguments:
#    type: the type of the cursor
#
# Results:
#    none returned.
#----------------------------------------------------------
#
proc setCursor {{type NORMAL}} {
    if {[lsearch -exact "CLOCK WAIT WATCH" [string toupper $type]] != -1} {
        set type watch
    } else {
        set type left_ptr
    }
    foreach wn [winfo children .] {
        catch {$wn configure -cursor $type}
    }
    update idletasks
}

#----------------------------------------------------------
# Window --
#
#    Window handling fuction
#
# Arguments:
#    args: [command name newname rest] list of arguments
#
# Results:
#    none returned.
#----------------------------------------------------------
#
proc Window {args} {
    global vTcl
    set cmd [lindex $args 0]
    set name [lindex $args 1]
    set newname [lindex $args 2]
    set rest [lrange $args 3 end]
    if {$name == "" || $cmd == ""} {return}
    if {$newname == ""} {
        set newname $name
    }
    set exists [winfo exists $newname]
    switch -- $cmd {
        show {
            if {$exists == "1" && $name != "."} {wm deiconify $name; return}
            if {[info procs vTclWindow(pre)$name] != ""} {
                eval "vTclWindow(pre)$name $newname $rest"
            }
            if {[info procs vTclWindow$name] != ""} {
                eval "vTclWindow$name $newname $rest"
            }
            if {[info procs vTclWindow(post)$name] != ""} {
                eval "vTclWindow(post)$name $newname $rest"
            }
        }
        hide    { if $exists {wm withdraw $newname; return} }
        iconify { if $exists {wm iconify $newname; return} }
        destroy { if $exists {destroy $newname; return} }
    }
}

#----------------------------------------------------------
# vTclWindow. --
#
#    Handles the main window
#
# Arguments:
#    base: the name of the window "."
#
# Results:
#    none returned.
#----------------------------------------------------------
#
proc vTclWindow. {base} {
    if {$base == ""} {
        set base .
    }
    wm focusmodel $base passive
    wm geometry $base 1x1+0+0
    wm maxsize $base 1009 738
    wm minsize $base 1 1
    wm overrideredirect $base 0
    wm resizable $base 1 1
    wm withdraw $base
    wm title $base "vt.tcl"
}
